@@ -10,13 +10,20 @@
 #define HAVE_FCNTL_H
 #include <io.h>
 #else
-#include <config.h>
+#include "config.h"
 #endif
+
 #include <stdio.h>
-#include <ctype.h>
 #include <string.h> /* for memset() only */
+#ifdef HAVE_CTYPE_H
+#include <ctype.h>
+#endif
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
+#endif
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
@@ -27,12 +34,17 @@
 #include <zlib.h>
 #endif
 
+#include "xmlmemory.h"
 #include "tree.h"
 #include "HTMLparser.h"
 #include "entities.h"
 #include "encoding.h"
 #include "valid.h"
 #include "parserInternals.h"
+#include "xmlIO.h"
+
+#define HTML_MAX_NAMELEN 1000
+#define INPUT_CHUNK     50
 
 /* #define DEBUG */
 
@@ -50,7 +62,7 @@
 int html##name##Push(htmlParserCtxtPtr ctxt, type value) {		\
     if (ctxt->name##Nr >= ctxt->name##Max) {				\
 	ctxt->name##Max *= 2;						\
-        ctxt->name##Tab = (void *) realloc(ctxt->name##Tab,		\
+        ctxt->name##Tab = (void *) xmlRealloc(ctxt->name##Tab,		\
 	             ctxt->name##Max * sizeof(ctxt->name##Tab[0]));	\
         if (ctxt->name##Tab == NULL) {					\
 	    fprintf(stderr, "realloc failed !\n");			\
@@ -101,7 +113,6 @@ PUSH_AND_POP(xmlNodePtr, node)
  *           UTF-8 if we are using this mode. It returns an int.
  *   NEXT    Skip to the next character, this does the proper decoding
  *           in UTF-8 mode. It also pop-up unfinished entities on the fly.
- *           It returns the pointer to the current CHAR.
  *   COPY(to) copy one char to *to, increment CUR_PTR and to accordingly
  */
 
@@ -111,21 +122,46 @@ PUSH_AND_POP(xmlNodePtr, node)
 #define NXT(val) ctxt->input->cur[(val)]
 #define UPP(val) (toupper(ctxt->input->cur[(val)]))
 #define CUR_PTR ctxt->input->cur
+#define SHRINK  xmlParserInputShrink(ctxt->input)
+#define GROW  xmlParserInputGrow(ctxt->input, INPUT_CHUNK)
 
 #define SKIP_BLANKS 							\
     while (IS_BLANK(*(ctxt->input->cur))) NEXT
 
 #ifndef USE_UTF_8
 #define CURRENT (*ctxt->input->cur)
+#define NEXT {								\
+    if ((*ctxt->input->cur == 0) &&					\
+        (xmlParserInputGrow(ctxt->input, INPUT_CHUNK) <= 0)) {		\
+	    xmlPopInput(ctxt);						\
+    } else {								\
+        if (*(ctxt->input->cur) == '\n') {				\
+	    ctxt->input->line++; ctxt->input->col = 1;			\
+	} else ctxt->input->col++;					\
+	ctxt->input->cur++;						\
+        if (*ctxt->input->cur == 0)					\
+	    xmlParserInputGrow(ctxt->input, INPUT_CHUNK);		\
+    }}
+
+/****************************************
 #define NEXT ((*ctxt->input->cur) ?					\
                 (((*(ctxt->input->cur) == '\n') ?			\
 		    (ctxt->input->line++, ctxt->input->col = 1) :	\
-		    (ctxt->input->col++)), ctxt->input->cur++) :	\
-		(ctxt->input->cur))
+		    (ctxt->input->col++)),				\
+		 (ctxt->input->cur++), 					\
+		 ((*ctxt->input->cur) ? 				\
+		  (xmlParserInputGrow(ctxt->input, 100),		\
+		   ctxt->input->cur):					\
+		  (ctxt->input->cur))) :		\
+		((xmlParserInputGrow(ctxt->input, 100) > 0) ?		\
+		 ctxt->input->cur:					\
+	         (xmlPopInput(ctxt), ctxt->input->cur)))
+ ****************************************/
 #else
 #endif
 
 
+
 /************************************************************************
  *									*
  * 		The list of HTML elements and their properties		*
@@ -241,7 +277,7 @@ htmlElemDesc  html40ElementTable[] = {
  * any tag of each line implies the end of the current element if the type of
  * that element is in the same line
  */
-CHAR *htmlEquEnd[] = {
+char *htmlEquEnd[] = {
 "DT", "DD", "LI", "OPTION", NULL,
 "H1", "H2", "H3", "H4", "H5", "H6", NULL,
 "OL", "MENU", "DIR", "ADDRESS", "PRE", "LISTING", "XMP", NULL,
@@ -256,7 +292,7 @@ NULL
 /*
  * start tags that imply the end of current element
  */
-CHAR *htmlStartClose[] = {
+char *htmlStartClose[] = {
 "FORM",		"FORM", "P", "HR", "H1", "H2", "H3", "H4", "H5", "H6",
 		"DL", "UL", "OL", "MENU", "DIR", "ADDRESS", "PRE",
 		"LISTING", "XMP", "HEAD", NULL,
@@ -310,7 +346,7 @@ CHAR *htmlStartClose[] = {
 NULL
 };
 
-static CHAR** htmlStartCloseIndex[100];
+static char** htmlStartCloseIndex[100];
 static int htmlStartCloseIndexinitialized = 0;
 
 /************************************************************************
@@ -354,7 +390,7 @@ htmlTagLookup(const CHAR *tag) {
 
     for (i = 0; i < (sizeof(html40ElementTable) /
                      sizeof(html40ElementTable[0]));i++) {
-        if (!xmlStrcmp(tag, html40ElementTable[i].name))
+        if (!xmlStrcmp(tag, BAD_CAST html40ElementTable[i].name))
 	    return(&html40ElementTable[i]);
     }
     return(NULL);
@@ -373,7 +409,7 @@ htmlTagLookup(const CHAR *tag) {
 int
 htmlCheckAutoClose(const CHAR *new, const CHAR *old) {
     int i, index;
-    CHAR **close;
+    char **close;
 
     if (htmlStartCloseIndexinitialized == 0) htmlInitAutoClose();
 
@@ -381,13 +417,13 @@ htmlCheckAutoClose(const CHAR *new, const CHAR *old) {
     for (index = 0; index < 100;index++) {
         close = htmlStartCloseIndex[index];
 	if (close == NULL) return(0);
-	if (!xmlStrcmp(*close, new)) break;
+	if (!xmlStrcmp(BAD_CAST *close, new)) break;
     }
 
     i = close - htmlStartClose;
     i++;
     while (htmlStartClose[i] != NULL) {
-        if (!xmlStrcmp(htmlStartClose[i], old)) {
+        if (!xmlStrcmp(BAD_CAST htmlStartClose[i], old)) {
 	    return(1);
 	}
 	i++;
@@ -487,13 +523,11 @@ htmlEntityDesc  html40EntitiesTable[] = {
 { 180,	"acute","acute accent = spacing acute, U+00B4 ISOdia" },
 { 181,	"micro","micro sign, U+00B5 ISOnum" },
 { 182,	"para",	"pilcrow sign = paragraph sign, U+00B6 ISOnum" },
-{ 183,	"middot","middle dot = Georgian comma
-                                  = Greek middle dot, U+00B7 ISOnum" },
+{ 183,	"middot","middle dot = Georgian comma Greek middle dot, U+00B7 ISOnum" },
 { 184,	"cedil","cedilla = spacing cedilla, U+00B8 ISOdia" },
 { 185,	"sup1",	"superscript one = superscript digit one, U+00B9 ISOnum" },
 { 186,	"ordm",	"masculine ordinal indicator, U+00BA ISOnum" },
-{ 187,	"raquo","right-pointing double angle quotation mark
-                                  = right pointing guillemet, U+00BB ISOnum" },
+{ 187,	"raquo","right-pointing double angle quotation mark right pointing guillemet, U+00BB ISOnum" },
 { 188,	"frac14","vulgar fraction one quarter = fraction one quarter, U+00BC ISOnum" },
 { 189,	"frac12","vulgar fraction one half = fraction one half, U+00BD ISOnum" },
 { 190,	"frac34","vulgar fraction three quarters = fraction three quarters, U+00BE ISOnum" },
@@ -522,7 +556,7 @@ htmlEntityDesc  html40EntitiesTable[] = {
 { 213,	"Otilde","latin capital letter O with tilde, U+00D5 ISOlat1" },
 { 214,	"Ouml",	"latin capital letter O with diaeresis, U+00D6 ISOlat1" },
 { 215,	"times","multiplication sign, U+00D7 ISOnum" },
-{ 216,	"Oslash","latin capital letter O with stroke = latin capital letter O slash, U+00D8 ISOlat1" },
+{ 216,	"Oslash","latin capital letter O with stroke latin capital letter O slash, U+00D8 ISOlat1" },
 { 217,	"Ugrave","latin capital letter U with grave, U+00D9 ISOlat1" },
 { 218,	"Uacute","latin capital letter U with acute, U+00DA ISOlat1" },
 { 219,	"Ucirc","latin capital letter U with circumflex, U+00DB ISOlat1" },
@@ -629,8 +663,7 @@ htmlEntityDesc  html40EntitiesTable[] = {
 { 8254,	"oline","overline = spacing overscore, U+203E NEW" },
 { 8260,	"frasl","fraction slash, U+2044 NEW" },
 
-{ 8472,	"weierp","script capital P = power set
-                                     = Weierstrass p, U+2118 ISOamso" },
+{ 8472,	"weierp","script capital P = power set = Weierstrass p, U+2118 ISOamso" },
 { 8465,	"image","blackletter capital I = imaginary part, U+2111 ISOamso" },
 { 8476,	"real",	"blackletter capital R = real part symbol, U+211C ISOamso" },
 { 8482,	"trade","trade mark sign, U+2122 ISOnum" },
@@ -726,8 +759,7 @@ htmlEntityDesc  html40EntitiesTable[] = {
 { 8225,	"Dagger","double dagger, U+2021 ISOpub" },
 { 8240,	"permil","per mille sign, U+2030 ISOtech" },
 { 8249,	"lsaquo","single left-pointing angle quotation mark, U+2039 ISO proposed" },
-{ 8250,	"rsaquo","single right-pointing angle quotation mark,
-                                    U+203A ISO proposed" },
+{ 8250,	"rsaquo","single right-pointing angle quotation mark, U+203A ISO proposed" },
 { 8364,	"euro",	"euro sign, U+20AC NEW" }
 };
 
@@ -742,7 +774,7 @@ htmlEntityDesc  html40EntitiesTable[] = {
  */
 #define growBuffer(buffer) {						\
     buffer##_size *= 2;							\
-    buffer = (CHAR *) realloc(buffer, buffer##_size * sizeof(CHAR));	\
+    buffer = (CHAR *) xmlRealloc(buffer, buffer##_size * sizeof(CHAR));	\
     if (buffer == NULL) {						\
 	perror("realloc failed");					\
 	exit(1);							\
@@ -765,7 +797,7 @@ htmlEntityLookup(const CHAR *name) {
 
     for (i = 0;i < (sizeof(html40EntitiesTable)/
                     sizeof(html40EntitiesTable[0]));i++) {
-        if (!xmlStrcmp(name, html40EntitiesTable[i].name)) {
+        if (!xmlStrcmp(name, BAD_CAST html40EntitiesTable[i].name)) {
 #ifdef DEBUG
             printf("Found entity %s\n", name);
 #endif
@@ -803,14 +835,14 @@ htmlDecodeEntities(htmlParserCtxtPtr ctxt, int len,
 
     CHAR *cur = NULL;
     htmlEntityDescPtr ent;
-    const CHAR *start = CUR_PTR;
+    int nbchars = 0;
     unsigned int max = (unsigned int) len;
 
     /*
      * allocate a translation buffer.
      */
     buffer_size = 1000;
-    buffer = (CHAR *) malloc(buffer_size * sizeof(CHAR));
+    buffer = (CHAR *) xmlMalloc(buffer_size * sizeof(CHAR));
     if (buffer == NULL) {
 	perror("htmlDecodeEntities: malloc failed");
 	return(NULL);
@@ -820,14 +852,15 @@ htmlDecodeEntities(htmlParserCtxtPtr ctxt, int len,
     /*
      * Ok loop until we reach one of the ending char or a size limit.
      */
-    while ((CUR_PTR - start < max) && (CUR != end) &&
+    while ((nbchars < max) && (CUR != end) &&
            (CUR != end2) && (CUR != end3)) {
 
         if (CUR == '&') {
 	    if (NXT(1) == '#') {
 		int val = htmlParseCharRef(ctxt);
-		/* TODO: invalid for UTF-8 variable encoding !!! */
+		/* invalid for UTF-8 variable encoding !!!!! */
 		*out++ = val;
+		nbchars += 3; /* !!!! */
 	    } else {
 		ent = htmlParseEntityRef(ctxt, &name);
 		if (name != NULL) {
@@ -846,7 +879,7 @@ htmlDecodeEntities(htmlParserCtxtPtr ctxt, int len,
 			}
 		        *out++ = ';';
 		    } else {
-			/* TODO: invalid for UTF-8 variable encoding !!! */
+			/* invalid for UTF-8 variable encoding !!!!! */
 			*out++ = (CHAR)ent->value;
 			if (out - buffer > buffer_size - 100) {
 			    int index = out - buffer;
@@ -855,12 +888,14 @@ htmlDecodeEntities(htmlParserCtxtPtr ctxt, int len,
 			    out = &buffer[index];
 			}
 		    }
-		    free(name);
+		    nbchars += 2 + xmlStrlen(name);
+		    xmlFree(name);
 		}
 	    }
 	} else {
-	    /*  TODO: invalid for UTF-8 , use COPY(out); */
+	    /*  invalid for UTF-8 , use COPY(out); !!!!! */
 	    *out++ = CUR;
+	    nbchars++;
 	    if (out - buffer > buffer_size - 100) {
 	      int index = out - buffer;
 	      
@@ -1023,8 +1058,6 @@ htmlSwitchEncoding(htmlParserCtxtPtr ctxt, xmlCharEncoding enc)
  *
  * Is this a sequence of blank chars that one can ignore ?
  *
- * TODO: to be corrected accodingly to DTD information if available
- *
  * Returns 1 if ignorable 0 otherwise.
  */
 
@@ -1089,16 +1122,17 @@ htmlNewDoc(const CHAR *URI, const CHAR *ExternalID) {
     /*
      * Allocate a new document and fill the fields.
      */
-    cur = (xmlDocPtr) malloc(sizeof(xmlDoc));
+    cur = (xmlDocPtr) xmlMalloc(sizeof(xmlDoc));
     if (cur == NULL) {
         fprintf(stderr, "xmlNewDoc : malloc failed\n");
 	return(NULL);
     }
+    memset(cur, 0, sizeof(xmlDoc));
 
     cur->type = XML_DOCUMENT_NODE;
     cur->version = NULL;
     cur->intSubset = NULL;
-    xmlCreateIntSubset(cur, "HTML", ExternalID, URI);
+    xmlCreateIntSubset(cur, BAD_CAST "HTML", ExternalID, URI);
     cur->name = NULL;
     cur->root = NULL; 
     cur->extSubset = NULL;
@@ -1106,6 +1140,8 @@ htmlNewDoc(const CHAR *URI, const CHAR *ExternalID) {
     cur->encoding = NULL;
     cur->standalone = 1;
     cur->compression = 0;
+    cur->ids = NULL;
+    cur->refs = NULL;
 #ifndef XML_WITHOUT_CORBA
     cur->_private = NULL;
     cur->vepv = NULL;
@@ -1170,49 +1206,67 @@ htmlParseHTMLName(htmlParserCtxtPtr ctxt) {
 
 CHAR *
 htmlParseName(htmlParserCtxtPtr ctxt) {
-    const CHAR *q;
-    CHAR *ret = NULL;
+    CHAR buf[HTML_MAX_NAMELEN];
+    int len = 0;
 
-    if (!IS_LETTER(CUR) && (CUR != '_') &&
-        (CUR != ':')) return(NULL);
-    q = NEXT;
+    GROW;
+    if (!IS_LETTER(CUR) && (CUR != '_')) {
+	return(NULL);
+    }
 
     while ((IS_LETTER(CUR)) || (IS_DIGIT(CUR)) ||
            (CUR == '.') || (CUR == '-') ||
 	   (CUR == '_') || (CUR == ':') || 
 	   (IS_COMBINING(CUR)) ||
-	   (IS_EXTENDER(CUR)))
+	   (IS_EXTENDER(CUR))) {
+	buf[len++] = CUR;
 	NEXT;
-    
-    ret = xmlStrndup(q, CUR_PTR - q);
-
-    return(ret);
+	if (len >= HTML_MAX_NAMELEN) {
+	    fprintf(stderr, 
+	       "htmlParseName: reached HTML_MAX_NAMELEN limit\n");
+	    while ((IS_LETTER(CUR)) || (IS_DIGIT(CUR)) ||
+		   (CUR == '.') || (CUR == '-') ||
+		   (CUR == '_') || (CUR == ':') || 
+		   (IS_COMBINING(CUR)) ||
+		   (IS_EXTENDER(CUR)))
+		 NEXT;
+	    break;
+	}
+    }
+    return(xmlStrndup(buf, len));
 }
 
 /**
  * htmlParseHTMLAttribute:
  * @ctxt:  an HTML parser context
  * 
- * parse an HTML Nmtoken.
+ * parse an HTML attribute value (without quotes).
  *
  * Returns the Nmtoken parsed or NULL
  */
 
 CHAR *
 htmlParseHTMLAttribute(htmlParserCtxtPtr ctxt) {
-    const CHAR *q;
-    CHAR *ret = NULL;
-
-    q = NEXT;
+    CHAR buf[HTML_MAX_NAMELEN];
+    int len = 0;
 
+    GROW;
     while ((!IS_BLANK(CUR)) && (CUR != '<') &&
            (CUR != '&') && (CUR != '>') &&
-           (CUR != '\'') && (CUR != '"'))
+           (CUR != '\'') && (CUR != '"')) {
+	buf[len++] = CUR;
 	NEXT;
-    
-    ret = xmlStrndup(q, CUR_PTR - q);
-
-    return(ret);
+	if (len >= HTML_MAX_NAMELEN) {
+	    fprintf(stderr, 
+	       "htmlParseHTMLAttribute: reached HTML_MAX_NAMELEN limit\n");
+	    while ((!IS_BLANK(CUR)) && (CUR != '<') &&
+		   (CUR != '&') && (CUR != '>') &&
+		   (CUR != '\'') && (CUR != '"'))
+		 NEXT;
+	    break;
+	}
+    }
+    return(xmlStrndup(buf, len));
 }
 
 /**
@@ -1226,21 +1280,30 @@ htmlParseHTMLAttribute(htmlParserCtxtPtr ctxt) {
 
 CHAR *
 htmlParseNmtoken(htmlParserCtxtPtr ctxt) {
-    const CHAR *q;
-    CHAR *ret = NULL;
-
-    q = NEXT;
+    CHAR buf[HTML_MAX_NAMELEN];
+    int len = 0;
 
+    GROW;
     while ((IS_LETTER(CUR)) || (IS_DIGIT(CUR)) ||
            (CUR == '.') || (CUR == '-') ||
 	   (CUR == '_') || (CUR == ':') || 
 	   (IS_COMBINING(CUR)) ||
-	   (IS_EXTENDER(CUR)))
+	   (IS_EXTENDER(CUR))) {
+	buf[len++] = CUR;
 	NEXT;
-    
-    ret = xmlStrndup(q, CUR_PTR - q);
-
-    return(ret);
+	if (len >= HTML_MAX_NAMELEN) {
+	    fprintf(stderr, 
+	       "htmlParseNmtoken: reached HTML_MAX_NAMELEN limit\n");
+	    while ((IS_LETTER(CUR)) || (IS_DIGIT(CUR)) ||
+		   (CUR == '.') || (CUR == '-') ||
+		   (CUR == '_') || (CUR == ':') || 
+		   (IS_COMBINING(CUR)) ||
+		   (IS_EXTENDER(CUR)))
+		 NEXT;
+	    break;
+	}
+    }
+    return(xmlStrndup(buf, len));
 }
 
 /**
@@ -1269,6 +1332,7 @@ htmlParseEntityRef(htmlParserCtxtPtr ctxt, CHAR **str) {
 	        ctxt->sax->error(ctxt->userData, "htmlParseEntityRef: no name\n");
 	    ctxt->wellFormed = 0;
 	} else {
+	    GROW;
 	    if (CUR == ';') {
 	        NEXT;
 		*str = name;
@@ -1283,10 +1347,10 @@ htmlParseEntityRef(htmlParserCtxtPtr ctxt, CHAR **str) {
 		                     "htmlParseEntityRef: expecting ';'\n");
 		ctxt->wellFormed = 0;
 		if (ctxt->sax->characters != NULL) {
-		    ctxt->sax->characters(ctxt->userData, "&", 1);
+		    ctxt->sax->characters(ctxt->userData, BAD_CAST "&", 1);
 		    ctxt->sax->characters(ctxt->userData, name, xmlStrlen(name));
 		}
-		free(name);
+		xmlFree(name);
 	    }
 	}
     }
@@ -1565,36 +1629,10 @@ htmlParseExternalID(htmlParserCtxtPtr ctxt, CHAR **publicID, int strict) {
 	          "htmlParseExternalID: PUBLIC, no Public Identifier\n");
 	    ctxt->wellFormed = 0;
 	}
-	if (strict) {
-	    /*
-	     * We don't handle [83] so "S SystemLiteral" is required.
-	     */
-	    if (!IS_BLANK(CUR)) {
-		if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
-		    ctxt->sax->error(ctxt->userData,
-			"Space required after the Public Identifier\n");
-		ctxt->wellFormed = 0;
-	    }
-	} else {
-	    /*
-	     * We handle [83] so we return immediately, if 
-	     * "S SystemLiteral" is not detected. From a purely parsing
-	     * point of view that's a nice mess.
-	     */
-	    const CHAR *ptr = CUR_PTR;
-	    if (!IS_BLANK(*ptr)) return(NULL);
-	    
-	    while (IS_BLANK(*ptr)) ptr++;
-	    if ((*ptr != '\'') || (*ptr != '"')) return(NULL);
-	}
         SKIP_BLANKS;
-	URI = htmlParseSystemLiteral(ctxt);
-	if (URI == NULL) {
-	    if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
-	        ctxt->sax->error(ctxt->userData, 
-	           "htmlParseExternalID: PUBLIC, no URI\n");
-	    ctxt->wellFormed = 0;
-        }
+        if ((CUR == '"') || (CUR == '\'')) {
+	    URI = htmlParseSystemLiteral(ctxt);
+	}
     }
     return(URI);
 }
@@ -1646,7 +1684,7 @@ htmlParseComment(htmlParserCtxtPtr ctxt, int create) {
 	    val = xmlStrndup(start, q - start);
 	    if ((ctxt->sax != NULL) && (ctxt->sax->comment != NULL))
 		ctxt->sax->comment(ctxt->userData, val);
-	    free(val);
+	    xmlFree(val);
 	}
     }
 }
@@ -1766,7 +1804,7 @@ htmlParseDocTypeDecl(htmlParserCtxtPtr ctxt) {
     /*
      * Check for SystemID and ExternalID
      */
-    URI = htmlParseExternalID(ctxt, &ExternalID, 1);
+    URI = htmlParseExternalID(ctxt, &ExternalID, 0);
     SKIP_BLANKS;
 
     /*
@@ -1788,11 +1826,10 @@ htmlParseDocTypeDecl(htmlParserCtxtPtr ctxt) {
 
     /*
      * Cleanup, since we don't use all those identifiers
-     * TODO : the DOCTYPE if available should be stored !
      */
-    if (URI != NULL) free(URI);
-    if (ExternalID != NULL) free(ExternalID);
-    if (name != NULL) free(name);
+    if (URI != NULL) xmlFree(URI);
+    if (ExternalID != NULL) xmlFree(ExternalID);
+    if (name != NULL) xmlFree(name);
 }
 
 /**
@@ -1918,8 +1955,8 @@ htmlParseStartTag(htmlParserCtxtPtr ctxt) {
 			ctxt->sax->error(ctxt->userData, "Attribute %s redefined\n",
 			                 name);
 		    ctxt->wellFormed = 0;
-		    free(attname);
-		    free(attvalue);
+		    xmlFree(attname);
+		    xmlFree(attvalue);
 		    break;
 		}
 	    }
@@ -1929,7 +1966,7 @@ htmlParseStartTag(htmlParserCtxtPtr ctxt) {
 	     */
 	    if (atts == NULL) {
 	        maxatts = 10;
-	        atts = (const CHAR **) malloc(maxatts * sizeof(CHAR *));
+	        atts = (const CHAR **) xmlMalloc(maxatts * sizeof(CHAR *));
 		if (atts == NULL) {
 		    fprintf(stderr, "malloc of %ld byte failed\n",
 			    maxatts * (long)sizeof(CHAR *));
@@ -1937,7 +1974,7 @@ htmlParseStartTag(htmlParserCtxtPtr ctxt) {
 		}
 	    } else if (nbatts + 2 < maxatts) {
 	        maxatts *= 2;
-	        atts = (const CHAR **) realloc(atts, maxatts * sizeof(CHAR *));
+	        atts = (const CHAR **) xmlRealloc(atts, maxatts * sizeof(CHAR *));
 		if (atts == NULL) {
 		    fprintf(stderr, "realloc of %ld byte failed\n",
 			    maxatts * (long)sizeof(CHAR *));
@@ -1967,8 +2004,8 @@ htmlParseStartTag(htmlParserCtxtPtr ctxt) {
         ctxt->sax->startElement(ctxt->userData, name, atts);
 
     if (atts != NULL) {
-        for (i = 0;i < nbatts;i++) free((CHAR *) atts[i]);
-	free(atts);
+        for (i = 0;i < nbatts;i++) xmlFree((CHAR *) atts[i]);
+	xmlFree(atts);
     }
     return(name);
 }
@@ -2027,6 +2064,7 @@ htmlParseEndTag(htmlParserCtxtPtr ctxt, const CHAR *tagname) {
 	    ctxt->sax->error(ctxt->userData, 
 	                     "htmlParseEndTag: unexpected close for tag %s\n",
 			     tagname);
+	xmlFree(name);
 	ctxt->wellFormed = 0;
 	return;
     }
@@ -2059,7 +2097,7 @@ htmlParseEndTag(htmlParserCtxtPtr ctxt, const CHAR *tagname) {
         ctxt->sax->endElement(ctxt->userData, name);
 
     if (name != NULL)
-	free(name);
+	xmlFree(name);
 
     return;
 }
@@ -2083,7 +2121,7 @@ htmlParseReference(htmlParserCtxtPtr ctxt) {
 
     if (NXT(1) == '#') {
 	val = htmlParseCharRef(ctxt);
-	/* TODO: invalid for UTF-8 variable encoding !!! */
+	/* invalid for UTF-8 variable encoding !!!!! */
 	out[0] = val;
 	out[1] = 0;
 	if ((ctxt->sax != NULL) && (ctxt->sax->characters != NULL))
@@ -2093,18 +2131,18 @@ htmlParseReference(htmlParserCtxtPtr ctxt) {
 	if (name == NULL) return; /* Shall we output & anyway ? */
 	if ((ent == NULL) || (ent->value <= 0) || (ent->value >= 255)) {
 	    if ((ctxt->sax != NULL) && (ctxt->sax->characters != NULL)) {
-		ctxt->sax->characters(ctxt->userData, "&", 1);
+		ctxt->sax->characters(ctxt->userData, BAD_CAST "&", 1);
 		ctxt->sax->characters(ctxt->userData, name, xmlStrlen(name));
-		ctxt->sax->characters(ctxt->userData, ";", 1);
+		ctxt->sax->characters(ctxt->userData, BAD_CAST ";", 1);
 	    }
 	} else {
-	    /* TODO: invalid for UTF-8 variable encoding !!! */
+	    /* invalid for UTF-8 variable encoding !!!!! */
 	    out[0] = ent->value;
 	    out[1] = 0;
 	    if ((ctxt->sax != NULL) && (ctxt->sax->characters != NULL))
 		ctxt->sax->characters(ctxt->userData, out, 1);
 	}
-	free(name);
+	xmlFree(name);
     }
 }
 
@@ -2168,6 +2206,7 @@ htmlParseContent(htmlParserCtxtPtr ctxt, const CHAR *name) {
 	    ctxt->wellFormed = 0;
             break;
 	}
+        GROW;
     }
 
     /*
@@ -2191,18 +2230,22 @@ void
 htmlParseElement(htmlParserCtxtPtr ctxt) {
     const CHAR *openTag = CUR_PTR;
     CHAR *name;
-    htmlParserNodeInfo node_info;
     htmlNodePtr currentNode;
     htmlElemDescPtr info;
+    htmlParserNodeInfo node_info;
 
     /* Capture start position */
-    node_info.begin_pos = CUR_PTR - ctxt->input->base;
-    node_info.begin_line = ctxt->input->line;
+    if (ctxt->record_info) {
+        node_info.begin_pos = ctxt->input->consumed +
+                          (CUR_PTR - ctxt->input->base);
+	node_info.begin_line = ctxt->input->line;
+    }
 
     name = htmlParseStartTag(ctxt);
     if (name == NULL) {
         return;
     }
+    currentNode = ctxt->node;
 
     /*
      * Lookup the info for that element.
@@ -2228,12 +2271,13 @@ htmlParseElement(htmlParserCtxtPtr ctxt) {
         SKIP(2);
 	if ((ctxt->sax != NULL) && (ctxt->sax->endElement != NULL))
 	    ctxt->sax->endElement(ctxt->userData, name);
-	free(name);
+	xmlFree(name);
 	return;
     }
 
-    if (CUR == '>') NEXT;
-    else {
+    if (CUR == '>') {
+        NEXT;
+    } else {
 	if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
 	    ctxt->sax->error(ctxt->userData, "Couldn't find end of Start Tag\n%.30s\n",
 	                     openTag);
@@ -2243,7 +2287,18 @@ htmlParseElement(htmlParserCtxtPtr ctxt) {
 	 * end of parsing of this node.
 	 */
 	nodePop(ctxt);
-	free(name);
+	xmlFree(name);
+
+	/*
+	 * Capture end position and add node
+	 */
+	if ( currentNode != NULL && ctxt->record_info ) {
+	   node_info.end_pos = ctxt->input->consumed +
+			      (CUR_PTR - ctxt->input->base);
+	   node_info.end_line = ctxt->input->line;
+	   node_info.node = currentNode;
+	   xmlParserAddNodeInfo(ctxt, &node_info);
+	}
 	return;
     }
 
@@ -2253,7 +2308,7 @@ htmlParseElement(htmlParserCtxtPtr ctxt) {
     if ((info != NULL) && (info->empty)) {
 	if ((ctxt->sax != NULL) && (ctxt->sax->endElement != NULL))
 	    ctxt->sax->endElement(ctxt->userData, name);
-	free(name);
+	xmlFree(name);
 	return;
     }
 
@@ -2268,7 +2323,7 @@ htmlParseElement(htmlParserCtxtPtr ctxt) {
      * on start tag
      */
     if (currentNode != ctxt->node) {
-	free(name);
+	xmlFree(name);
         return;
     }
 
@@ -2282,11 +2337,22 @@ htmlParseElement(htmlParserCtxtPtr ctxt) {
 	 * end of parsing of this node.
 	 */
 	nodePop(ctxt);
-	free(name);
+	xmlFree(name);
 	return;
     }
 
-    free(name);
+    xmlFree(name);
+
+    /*
+     * Capture end position and add node
+     */
+    if ( currentNode != NULL && ctxt->record_info ) {
+       node_info.end_pos = ctxt->input->consumed +
+                          (CUR_PTR - ctxt->input->base);
+       node_info.end_line = ctxt->input->line;
+       node_info.node = currentNode;
+       xmlParserAddNodeInfo(ctxt, &node_info);
+    }
 }
 
 /**
@@ -2305,17 +2371,13 @@ htmlParseDocument(htmlParserCtxtPtr ctxt) {
     htmlDefaultSAXHandlerInit();
     ctxt->html = 1;
 
+    GROW;
     /*
-     * SAX: beginning of the document processing TODO: update for HTML.
+     * SAX: beginning of the document processing.
      */
     if ((ctxt->sax) && (ctxt->sax->setDocumentLocator))
         ctxt->sax->setDocumentLocator(ctxt->userData, &xmlDefaultSAXLocator);
 
-    /*
-     * We should check for encoding here and plug-in some
-     * conversion code TODO !!!!
-     */
-
     /*
      * Wipe out everything which is before the first '<'
      */
@@ -2387,13 +2449,13 @@ htmlInitParserCtxt(htmlParserCtxtPtr ctxt)
 {
     htmlSAXHandler *sax;
 
-    sax = (htmlSAXHandler *) malloc(sizeof(htmlSAXHandler));
+    sax = (htmlSAXHandler *) xmlMalloc(sizeof(htmlSAXHandler));
     if (sax == NULL) {
         fprintf(stderr, "htmlInitParserCtxt: out of memory\n");
     }
 
     /* Allocate the Input stack */
-    ctxt->inputTab = (htmlParserInputPtr *) malloc(5 * sizeof(htmlParserInputPtr));
+    ctxt->inputTab = (htmlParserInputPtr *) xmlMalloc(5 * sizeof(htmlParserInputPtr));
     ctxt->inputNr = 0;
     ctxt->inputMax = 5;
     ctxt->input = NULL;
@@ -2402,7 +2464,7 @@ htmlInitParserCtxt(htmlParserCtxtPtr ctxt)
     ctxt->standalone = -1;
 
     /* Allocate the Node stack */
-    ctxt->nodeTab = (htmlNodePtr *) malloc(10 * sizeof(htmlNodePtr));
+    ctxt->nodeTab = (htmlNodePtr *) xmlMalloc(10 * sizeof(htmlNodePtr));
     ctxt->nodeNr = 0;
     ctxt->nodeMax = 10;
     ctxt->node = NULL;
@@ -2440,12 +2502,12 @@ htmlFreeParserCtxt(htmlParserCtxtPtr ctxt)
         xmlFreeInputStream(input);
     }
 
-    if (ctxt->nodeTab != NULL) free(ctxt->nodeTab);
-    if (ctxt->inputTab != NULL) free(ctxt->inputTab);
-    if (ctxt->version != NULL) free((char *) ctxt->version);
+    if (ctxt->nodeTab != NULL) xmlFree(ctxt->nodeTab);
+    if (ctxt->inputTab != NULL) xmlFree(ctxt->inputTab);
+    if (ctxt->version != NULL) xmlFree((char *) ctxt->version);
     if ((ctxt->sax != NULL) && (ctxt->sax != &htmlDefaultSAXHandler))
-        free(ctxt->sax);
-    free(ctxt);
+        xmlFree(ctxt->sax);
+    xmlFree(ctxt);
 }
 
 /**
@@ -2463,16 +2525,16 @@ htmlCreateDocParserCtxt(CHAR *cur, const char *encoding) {
     htmlParserInputPtr input;
     /* htmlCharEncoding enc; */
 
-    ctxt = (htmlParserCtxtPtr) malloc(sizeof(htmlParserCtxt));
+    ctxt = (htmlParserCtxtPtr) xmlMalloc(sizeof(htmlParserCtxt));
     if (ctxt == NULL) {
         perror("malloc");
 	return(NULL);
     }
     htmlInitParserCtxt(ctxt);
-    input = (htmlParserInputPtr) malloc(sizeof(htmlParserInput));
+    input = (htmlParserInputPtr) xmlMalloc(sizeof(htmlParserInput));
     if (input == NULL) {
         perror("malloc");
-	free(ctxt);
+	xmlFree(ctxt);
 	return(NULL);
     }
 
@@ -2490,6 +2552,7 @@ htmlCreateDocParserCtxt(CHAR *cur, const char *encoding) {
     input->base = cur;
     input->cur = cur;
     input->free = NULL;
+    input->buf = NULL;
 
     inputPush(ctxt, input);
     return(ctxt);
@@ -2572,143 +2635,34 @@ htmlParserCtxtPtr
 htmlCreateFileParserCtxt(const char *filename, const char *encoding)
 {
     htmlParserCtxtPtr ctxt;
-#ifdef HAVE_ZLIB_H
-    gzFile input;
-#else
-    int input;
-#endif
-    int res;
-    int len;
-    int cnt;
-    struct stat buf;
-    char *buffer, *nbuf;
     htmlParserInputPtr inputStream;
+    xmlParserInputBufferPtr buf;
     /* htmlCharEncoding enc; */
 
-#define MINLEN 40000
+    buf = xmlParserInputBufferCreateFilename(filename, XML_CHAR_ENCODING_NONE);
+    if (buf == NULL) return(NULL);
 
-    if (strcmp(filename,"-") == 0) {
-#ifdef HAVE_ZLIB_H
-        input = gzdopen (fileno(stdin), "r");
-        if (input == NULL) {
-            fprintf (stderr, "Cannot read from stdin\n");
-            perror ("gzdopen failed");
-            return(NULL);
-	}
-#else
-#ifdef WIN32
-        input = -1;
-#else
-        input = fileno(stdin);
-#endif
-	if (input < 0) {
-	    fprintf (stderr, "Cannot read from stdin\n");
-	    perror ("open failed");
-	    return(NULL);
-	}
-#endif
-	len = MINLEN;
-    } else {
-#ifdef HAVE_ZLIB_H
-	input = gzopen (filename, "r");
-	if (input == NULL) {
-	    fprintf (stderr, "Cannot read file %s :\n", filename);
-	    perror ("gzopen failed");
-	    return(NULL);
-	}
-#else
-#ifdef WIN32
-	input = _open (filename, O_RDONLY | _O_BINARY);
-#else
-	input = open (filename, O_RDONLY);
-#endif
-	if (input < 0) {
-	    fprintf (stderr, "Cannot read file %s :\n", filename);
-	    perror ("open failed");
-	    return(NULL);
-	}
-#endif
-	res = stat(filename, &buf);
-	if (res < 0)
-		return(NULL);
-	len = buf.st_size;
-	if (len < MINLEN)
-		len = MINLEN;
-    }
-    buffer = (char *)malloc((len+1)*sizeof(char));
-    if (buffer == NULL) {
-	    fprintf (stderr, "Cannot malloc\n");
-	    perror ("malloc failed");
-	    return(NULL);
-    }
-
-    cnt = 0;
-    while(1) {
-	if (cnt == len) {
-	    len *= 2;
-	    nbuf =  (char *)realloc(buffer,(len+1)*sizeof(char));
-	    if (nbuf == NULL) {
-		fprintf(stderr,"Cannot realloc\n");
-		free(buffer);
-		perror ("realloc failed");
-		return(NULL);
-	    }
-	    buffer = nbuf;
-	}
-#ifdef HAVE_ZLIB_H
-    	res = gzread(input, &buffer[cnt], len-cnt);
-#else
-	res = read(input, &buffer[cnt], len-cnt);
-#endif
-	if (res < 0) {
-	    fprintf (stderr, "Cannot read file %s :\n", filename);
-#ifdef HAVE_ZLIB_H
-	    perror ("gzread failed");
-#else
-	    perror ("read failed");
-#endif
-	    return(NULL);
-	}
-	if (res == 0)
-	    break;
-	cnt +=res;
-    }
-    buffer[cnt] = '\0';
-
-#ifdef HAVE_ZLIB_H
-    gzclose(input);
-#else
-    close(input);
-#endif
-
-    ctxt = (htmlParserCtxtPtr) malloc(sizeof(htmlParserCtxt));
+    ctxt = (htmlParserCtxtPtr) xmlMalloc(sizeof(htmlParserCtxt));
     if (ctxt == NULL) {
         perror("malloc");
 	return(NULL);
     }
     htmlInitParserCtxt(ctxt);
-    inputStream = (htmlParserInputPtr) malloc(sizeof(htmlParserInput));
+    inputStream = (htmlParserInputPtr) xmlMalloc(sizeof(htmlParserInput));
     if (inputStream == NULL) {
         perror("malloc");
-	free(ctxt);
+	xmlFree(ctxt);
 	return(NULL);
     }
 
-    inputStream->filename = strdup(filename);
+    inputStream->filename = xmlMemStrdup(filename);
     inputStream->line = 1;
     inputStream->col = 1;
+    inputStream->buf = buf;
 
-    /*
-     * plug some encoding conversion routines here. !!!
-    if (encoding != NULL) {
-	enc = htmlDetectCharEncoding(buffer);
-	htmlSwitchEncoding(ctxt, enc);
-    }
-     */
-
-    inputStream->base = buffer;
-    inputStream->cur = buffer;
-    inputStream->free = (xmlParserInputDeallocate) free;
+    inputStream->base = inputStream->buf->buffer->content;
+    inputStream->cur = inputStream->buf->buffer->content;
+    inputStream->free = NULL;
 
     inputPush(ctxt, inputStream);
     return(ctxt);
