@@ -6,12 +6,21 @@
  * Daniel.Veillard@w3.org
  */
 
+
+#ifndef WIN32
 #include "config.h"
+#endif
 #include <stdio.h>
-#include <ctype.h>
-#include <stdlib.h>
 #include <string.h> /* for memset() only ! */
 
+#ifdef HAVE_CTYPE_H
+#include <ctype.h>
+#endif
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+
+#include "xmlmemory.h"
 #include "HTMLparser.h"
 #include "HTMLtree.h"
 #include "entities.h"
@@ -70,7 +79,7 @@ htmlAttrDump(xmlBufferPtr buf, xmlDocPtr doc, xmlAttrPtr cur) {
     if (value) {
 	xmlBufferWriteChar(buf, "=");
 	xmlBufferWriteQuotedString(buf, value);
-	free(value);
+	xmlFree(value);
     } else  {
 	xmlBufferWriteChar(buf, "=\"\"");
     }
@@ -146,7 +155,7 @@ htmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur) {
             buffer = xmlEncodeEntitiesReentrant(doc, cur->content);
 	    if (buffer != NULL) {
 		xmlBufferWriteCHAR(buf, buffer);
-		free(buffer);
+		xmlFree(buffer);
 	    }
 	}
 	return;
@@ -207,7 +216,7 @@ htmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur) {
 	buffer = xmlEncodeEntitiesReentrant(doc, cur->content);
 	if (buffer != NULL) {
 	    xmlBufferWriteCHAR(buf, buffer);
-	    free(buffer);
+	    xmlFree(buffer);
 	}
     }
     if (cur->childs != NULL) {
@@ -277,7 +286,7 @@ htmlDocDumpMemory(xmlDocPtr cur, CHAR**mem, int *size) {
     *mem = buf->content;
     *size = buf->use;
     memset(buf, -1, sizeof(xmlBuffer));
-    free(buf);
+    xmlFree(buf);
 }
 
 
