@@ -6,16 +6,20 @@
  * Daniel Veillard <Daniel.Veillard@w3.org>
  */
 
+
 #include <stdio.h>
 #include <stdlib.h>
+#include "xmlmemory.h"
 #include "tree.h"
 #include "parser.h"
 #include "parserInternals.h"
 #include "valid.h"
 #include "entities.h"
 #include "xml-error.h"
+#include "debugXML.h"
 
 /* #define DEBUG_SAX */
+/* #define DEBUG_SAX_TREE */
 
 /**
  * getPublicId:
@@ -45,7 +49,7 @@ const CHAR *
 getSystemId(void *ctx)
 {
     xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) ctx;
-    return(ctxt->input->filename); 
+    return(BAD_CAST ctxt->input->filename); 
 }
 
 /**
@@ -132,7 +136,7 @@ hasExternalSubset(void *ctx)
 }
 
 /**
- * hasInternalSubset:
+ * internalSubset:
  * @ctx: the user data (XML parser context)
  *
  * Does this document has an internal subset
@@ -141,15 +145,72 @@ void
 internalSubset(void *ctx, const CHAR *name,
 	       const CHAR *ExternalID, const CHAR *SystemID)
 {
-    xmlDtdPtr externalSubset;
     xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) ctx;
 #ifdef DEBUG_SAX
     fprintf(stderr, "SAX.internalSubset(%s, %s, %s)\n",
             name, ExternalID, SystemID);
 #endif
     xmlCreateIntSubset(ctxt->myDoc, name, ExternalID, SystemID);
-    if ((ExternalID != NULL) || (SystemID != NULL)) {
-        externalSubset = xmlParseDTD(ExternalID, SystemID);
+    if (((ExternalID != NULL) || (SystemID != NULL)) &&
+        (ctxt->validate && ctxt->wellFormed && ctxt->myDoc)) {
+	/*
+	 * Try to fetch and parse the external subset.
+	 */
+	xmlDtdPtr ret = NULL;
+	xmlParserCtxtPtr dtdCtxt;
+	xmlParserInputPtr input = NULL;
+	xmlCharEncoding enc;
+
+	dtdCtxt = xmlNewParserCtxt();
+	if (dtdCtxt == NULL) return;
+
+	/*
+	 * Ask the Entity resolver to load the damn thing
+	 */
+	if ((ctxt->directory != NULL) && (dtdCtxt->directory == NULL))
+	    dtdCtxt->directory = (char *) xmlStrdup(BAD_CAST ctxt->directory);
+
+	if ((dtdCtxt->sax != NULL) && (dtdCtxt->sax->resolveEntity != NULL))
+	    input = dtdCtxt->sax->resolveEntity(dtdCtxt->userData, ExternalID,
+	                                        SystemID);
+	if (input == NULL) {
+	    xmlFreeParserCtxt(dtdCtxt);
+	    return;
+	}
+
+	/*
+	 * plug some encoding conversion routines here. !!!
+	 */
+	xmlPushInput(dtdCtxt, input);
+	enc = xmlDetectCharEncoding(dtdCtxt->input->cur);
+	xmlSwitchEncoding(dtdCtxt, enc);
+
+	if (input->filename == NULL)
+	    input->filename = (char *) xmlStrdup(SystemID);
+	input->line = 1;
+	input->col = 1;
+	input->base = dtdCtxt->input->cur;
+	input->cur = dtdCtxt->input->cur;
+	input->free = NULL;
+
+	/*
+	 * let's parse that entity knowing it's an external subset.
+	 */
+	xmlParseExternalSubset(dtdCtxt, ExternalID, SystemID);
+
+	if (dtdCtxt->myDoc != NULL) {
+	    if (dtdCtxt->wellFormed) {
+		ret = dtdCtxt->myDoc->intSubset;
+		dtdCtxt->myDoc->intSubset = NULL;
+	    } else {
+		ret = NULL;
+	    }
+	    xmlFreeDoc(dtdCtxt->myDoc);
+	    dtdCtxt->myDoc = NULL;
+	}
+	xmlFreeParserCtxt(dtdCtxt);
+	
+	ctxt->myDoc->extSubset = ret;
     }
 }
 
@@ -176,17 +237,8 @@ resolveEntity(void *ctx, const CHAR *publicId, const CHAR *systemId)
     fprintf(stderr, "SAX.resolveEntity(%s, %s)\n", publicId, systemId);
 #endif
 
-    /*
-     * TODO : not 100% sure that the appropriate handling in that case.
-     */
     if (systemId != NULL) {
-        if (!xmlStrncmp(systemId, "http://", 7)) {
-	    /* !!!!!!!!! TODO */
-	} else if (!xmlStrncmp(systemId, "ftp://", 6)) {
-	    /* !!!!!!!!! TODO */
-	} else {
-	    return(xmlNewInputFromFile(ctxt, systemId));
-	}
+	return(xmlNewInputFromFile(ctxt, (char *) systemId));
     }
     return(NULL);
 }
@@ -214,6 +266,29 @@ getEntity(void *ctx, const CHAR *name)
     return(ret);
 }
 
+/**
+ * getParameterEntity:
+ * @ctx: the user data (XML parser context)
+ * @name: The entity name
+ *
+ * Get a parameter entity by name
+ *
+ * Returns the xmlEntityPtr if found.
+ */
+xmlEntityPtr
+getParameterEntity(void *ctx, const CHAR *name)
+{
+    xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) ctx;
+    xmlEntityPtr ret;
+
+#ifdef DEBUG_SAX
+    fprintf(stderr, "SAX.getParameterEntity(%s)\n", name);
+#endif
+
+    ret = xmlGetParameterEntity(ctxt->myDoc, name);
+    return(ret);
+}
+
 
 /**
  * entityDecl:
@@ -256,13 +331,19 @@ attributeDecl(void *ctx, const CHAR *elem, const CHAR *name,
 	      xmlEnumerationPtr tree)
 {
     xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) ctx;
+    xmlAttributePtr attr;
 
 #ifdef DEBUG_SAX
     fprintf(stderr, "SAX.attributeDecl(%s, %s, %d, %d, %s, ...)\n",
             elem, name, type, def, defaultValue);
 #endif
-    xmlAddAttributeDecl(ctxt->myDoc->intSubset, elem, name, type, def,
-                        defaultValue, tree);
+    attr = xmlAddAttributeDecl(&ctxt->vctxt, ctxt->myDoc->intSubset, elem,
+                               name, type, def, defaultValue, tree);
+    if (attr == 0) ctxt->valid = 0;
+    if (ctxt->validate && ctxt->wellFormed &&
+        ctxt->myDoc && ctxt->myDoc->intSubset)
+	ctxt->valid &= xmlValidateAttributeDecl(&ctxt->vctxt, ctxt->myDoc,
+	                                        attr);
 }
 
 /**
@@ -281,12 +362,19 @@ elementDecl(void *ctx, const CHAR *name, int type,
 	    xmlElementContentPtr content)
 {
     xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) ctx;
+    xmlElementPtr elem;
 
 #ifdef DEBUG_SAX
     fprintf(stderr, "SAX.elementDecl(%s, %d, ...)\n",
             name, type);
 #endif
-    xmlAddElementDecl(ctxt->myDoc->intSubset, name, type, content);
+    
+    elem = xmlAddElementDecl(&ctxt->vctxt, ctxt->myDoc->intSubset,
+                             name, type, content);
+    if (elem == 0) ctxt->valid = 0;
+    if (ctxt->validate && ctxt->wellFormed &&
+        ctxt->myDoc && ctxt->myDoc->intSubset)
+	ctxt->valid &= xmlValidateElementDecl(&ctxt->vctxt, ctxt->myDoc, elem);
 }
 
 /**
@@ -297,17 +385,25 @@ elementDecl(void *ctx, const CHAR *name, int type,
  * @systemId: The system ID of the entity
  *
  * What to do when a notation declaration has been parsed.
- * TODO Not handled currently.
  */
 void
 notationDecl(void *ctx, const CHAR *name,
 	     const CHAR *publicId, const CHAR *systemId)
 {
     xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) ctx;
+    xmlNotationPtr nota;
+
 #ifdef DEBUG_SAX
     fprintf(stderr, "SAX.notationDecl(%s, %s, %s)\n", name, publicId, systemId);
 #endif
-    xmlAddNotationDecl(ctxt->myDoc->intSubset, name, publicId, systemId);
+
+    nota = xmlAddNotationDecl(&ctxt->vctxt, ctxt->myDoc->intSubset, name,
+                              publicId, systemId);
+    if (nota == 0) ctxt->valid = 0;
+    if (ctxt->validate && ctxt->wellFormed &&
+        ctxt->myDoc && ctxt->myDoc->intSubset)
+	ctxt->valid &= xmlValidateNotationDecl(&ctxt->vctxt, ctxt->myDoc,
+	                                       nota);
 }
 
 /**
@@ -319,18 +415,24 @@ notationDecl(void *ctx, const CHAR *name,
  * @notationName: the name of the notation
  *
  * What to do when an unparsed entity declaration is parsed
- * TODO Create an Entity node.
  */
 void
 unparsedEntityDecl(void *ctx, const CHAR *name,
 		   const CHAR *publicId, const CHAR *systemId,
 		   const CHAR *notationName)
 {
-    /* xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) ctx; */
+    xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) ctx;
 #ifdef DEBUG_SAX
     fprintf(stderr, "SAX.unparsedEntityDecl(%s, %s, %s, %s)\n",
             name, publicId, systemId, notationName);
 #endif
+    if (ctxt->validate && ctxt->wellFormed &&
+        ctxt->myDoc && ctxt->myDoc->intSubset)
+	ctxt->valid &= xmlValidateNotationUse(&ctxt->vctxt, ctxt->myDoc,
+	                                      notationName);
+    xmlAddDocEntity(ctxt->myDoc, name,
+                    XML_EXTERNAL_GENERAL_UNPARSED_ENTITY,
+		    publicId, systemId, notationName);
 }
 
 /**
@@ -384,10 +486,13 @@ startDocument(void *ctx)
 void
 endDocument(void *ctx)
 {
-    /* xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) ctx; */
+    xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) ctx;
 #ifdef DEBUG_SAX
     fprintf(stderr, "SAX.endDocument()\n");
 #endif
+    if (ctxt->validate && ctxt->wellFormed &&
+        ctxt->myDoc && ctxt->myDoc->intSubset)
+	ctxt->valid &= xmlValidateDocumentFinal(&ctxt->vctxt, ctxt->myDoc);
 }
 
 /**
@@ -408,6 +513,7 @@ attribute(void *ctx, const CHAR *fullname, const CHAR *value)
     xmlAttrPtr ret;
     CHAR *name;
     CHAR *ns;
+    xmlNsPtr namespace;
 
 /****************
 #ifdef DEBUG_SAX
@@ -428,26 +534,49 @@ attribute(void *ctx, const CHAR *fullname, const CHAR *value)
 	/* a default namespace definition */
 	xmlNewNs(ctxt->node, value, NULL);
 	if (name != NULL) 
-	    free(name);
+	    xmlFree(name);
 	return;
     }
     if ((ns != NULL) && (ns[0] == 'x') && (ns[1] == 'm') && (ns[2] == 'l') &&
         (ns[3] == 'n') && (ns[4] == 's') && (ns[5] == 0)) {
 	/* a standard namespace definition */
 	xmlNewNs(ctxt->node, value, name);
-	free(ns);
+	xmlFree(ns);
 	if (name != NULL) 
-	    free(name);
+	    xmlFree(name);
 	return;
     }
 
-    ret = xmlNewProp(ctxt->node, name, NULL);
-    if ((ret != NULL) && (ctxt->replaceEntities == 0))
-	ret->val = xmlStringGetNodeList(ctxt->myDoc, value);
+    namespace = xmlSearchNs(ctxt->myDoc, ctxt->node, ns);
+    /* !!!!!! <a toto:arg="" xmlns:toto="http://toto.com"> */
+    ret = xmlNewNsProp(ctxt->node, namespace, name, NULL);
+
+    if (ret != NULL) {
+        if (ctxt->replaceEntities == 0)
+	    ret->val = xmlStringGetNodeList(ctxt->myDoc, value);
+	else
+	    ret->val = xmlNewDocText(ctxt->myDoc, value);
+    }
+
+    if (ctxt->validate && ctxt->wellFormed &&
+        ctxt->myDoc && ctxt->myDoc->intSubset)
+        ctxt->valid &= xmlValidateOneAttribute(&ctxt->vctxt, ctxt->myDoc,
+					       ctxt->node, ret, value);
+    else {
+        /*
+	 * when validating, the ID registration is done at the attribute
+	 * validation level. Otherwise we have to do specific handling here.
+	 */
+	if (xmlIsID(ctxt->myDoc, ctxt->node, ret))
+	    xmlAddID(&ctxt->vctxt, ctxt->myDoc, value, ret);
+	else if (xmlIsRef(ctxt->myDoc, ctxt->node, ret))
+	    xmlAddRef(&ctxt->vctxt, ctxt->myDoc, value, ret);
+    }
+
     if (name != NULL) 
-	free(name);
+	xmlFree(name);
     if (ns != NULL) 
-	free(ns);
+	xmlFree(ns);
 }
 
 /**
@@ -457,7 +586,6 @@ attribute(void *ctx, const CHAR *fullname, const CHAR *value)
  * @atts:  An array of name/value attributes pairs, NULL terminated
  *
  * called when an opening tag has been processed.
- * TODO We currently have a small pblm with the arguments ...
  */
 void
 startElement(void *ctx, const CHAR *fullname, const CHAR **atts)
@@ -470,12 +598,29 @@ startElement(void *ctx, const CHAR *fullname, const CHAR **atts)
     CHAR *prefix;
     const CHAR *att;
     const CHAR *value;
-
     int i;
 
 #ifdef DEBUG_SAX
     fprintf(stderr, "SAX.startElement(%s)\n", fullname);
 #endif
+
+    /*
+     * First check on validity:
+     */
+    if (ctxt->validate && (ctxt->myDoc->extSubset == NULL) && 
+        ((ctxt->myDoc->intSubset == NULL) ||
+	 ((ctxt->myDoc->intSubset->notations == NULL) && 
+	  (ctxt->myDoc->intSubset->elements == NULL) &&
+	  (ctxt->myDoc->intSubset->attributes == NULL) && 
+	  (ctxt->myDoc->intSubset->entities == NULL)))) {
+	if (ctxt->vctxt.error != NULL) {
+            ctxt->vctxt.error(ctxt->vctxt.userData,
+	      "Validation failed: no DTD found !\n");
+	}
+	ctxt->validate = 0;
+    }
+       
+
     /*
      * Split the full name into a namespace prefix and the tag name
      */
@@ -489,32 +634,67 @@ startElement(void *ctx, const CHAR *fullname, const CHAR **atts)
      */
     ret = xmlNewDocNode(ctxt->myDoc, NULL, name, NULL);
     if (ret == NULL) return;
-    if (ctxt->myDoc->root == NULL)
+    if (ctxt->myDoc->root == NULL) {
+#ifdef DEBUG_SAX_TREE
+	fprintf(stderr, "Setting %s as root\n", name);
+#endif
         ctxt->myDoc->root = ret;
+    } else if (parent == NULL) {
+        parent = ctxt->myDoc->root;
+    }
 
     /*
      * We are parsing a new node.
      */
+#ifdef DEBUG_SAX_TREE
+    fprintf(stderr, "pushing(%s)\n", name);
+#endif
     nodePush(ctxt, ret);
 
     /*
      * Link the child element
      */
-    if (parent != NULL)
-	xmlAddChild(parent, ctxt->node);
+    if (parent != NULL) {
+        if (parent->type == XML_ELEMENT_NODE) {
+#ifdef DEBUG_SAX_TREE
+	    fprintf(stderr, "adding child %s to %s\n", name, parent->name);
+#endif
+	    xmlAddChild(parent, ret);
+	} else {
+#ifdef DEBUG_SAX_TREE
+	    fprintf(stderr, "adding sibling %s to ", name);
+	    xmlDebugDumpOneNode(stderr, parent, 0);
+#endif
+	    xmlAddSibling(parent, ret);
+	}
+    }
 
     /*
-     * process all the attributes.
+     * process all the attributes whose name start with "xml"
      */
     if (atts != NULL) {
         i = 0;
 	att = atts[i++];
 	value = atts[i++];
         while ((att != NULL) && (value != NULL)) {
-	    /*
-	     * Handle one pair of attribute/value
-	     */
-	    attribute(ctxt, att, value);
+	    if ((att[0] == 'x') && (att[1] == 'm') && (att[2] == 'l'))
+		attribute(ctxt, att, value);
+
+	    att = atts[i++];
+	    value = atts[i++];
+	}
+    }
+
+    /*
+     * process all the other attributes
+     */
+    if (atts != NULL) {
+        i = 0;
+	att = atts[i++];
+	value = atts[i++];
+        while ((att != NULL) && (value != NULL)) {
+	    if ((att[0] != 'x') || (att[1] != 'm') || (att[2] != 'l'))
+		attribute(ctxt, att, value);
 
 	    /*
 	     * Next ones
@@ -534,9 +714,9 @@ startElement(void *ctx, const CHAR *fullname, const CHAR **atts)
     xmlSetNs(ret, ns);
 
     if (prefix != NULL)
-	free(prefix);
+	xmlFree(prefix);
     if (name != NULL)
-	free(name);
+	xmlFree(name);
 
 }
 
@@ -569,9 +749,18 @@ endElement(void *ctx, const CHAR *name)
       xmlParserAddNodeInfo(ctxt, &node_info);
     }
 
+    if (ctxt->validate && ctxt->wellFormed &&
+        ctxt->myDoc && ctxt->myDoc->intSubset)
+        ctxt->valid &= xmlValidateOneElement(&ctxt->vctxt, ctxt->myDoc,
+					     cur);
+
+    
     /*
      * end of parsing of this node.
      */
+#ifdef DEBUG_SAX_TREE
+    fprintf(stderr, "popping(%s)\n", cur->name);
+#endif
     nodePop(ctxt);
 }
 
@@ -592,6 +781,9 @@ reference(void *ctx, const CHAR *name)
     fprintf(stderr, "SAX.reference(%s)\n", name);
 #endif
     ret = xmlNewReference(ctxt->myDoc, name);
+#ifdef DEBUG_SAX_TREE
+    fprintf(stderr, "add reference %s to %s \n", name, ctxt->node->name);
+#endif
     xmlAddChild(ctxt->node, ret);
 }
 
@@ -620,6 +812,9 @@ characters(void *ctx, const CHAR *ch, int len)
      */
 
     lastChild = xmlGetLastChild(ctxt->node);
+#ifdef DEBUG_SAX_TREE
+    fprintf(stderr, "add chars to %s \n", ctxt->node->name);
+#endif
     if (lastChild == NULL)
 	xmlNodeAddContentLen(ctxt->node, ch, len);
     else {
@@ -663,10 +858,40 @@ void
 processingInstruction(void *ctx, const CHAR *target,
                       const CHAR *data)
 {
-    /* xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) ctx; */
+    xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) ctx;
+    xmlNodePtr ret;
+    xmlNodePtr parent = ctxt->node;
+
 #ifdef DEBUG_SAX
     fprintf(stderr, "SAX.processingInstruction(%s, %s)\n", target, data);
 #endif
+
+    ret = xmlNewPI(target, data);
+    if (ret == NULL) return;
+    ret->doc = ctxt->myDoc;
+    if (ctxt->myDoc->root == NULL) {
+#ifdef DEBUG_SAX_TREE
+	    fprintf(stderr, "Setting PI %s as root\n", target);
+#endif
+        ctxt->myDoc->root = ret;
+    } else if (parent == NULL) {
+        parent = ctxt->myDoc->root;
+    }
+    if (parent != NULL) {
+        if (parent->type == XML_ELEMENT_NODE) {
+#ifdef DEBUG_SAX_TREE
+	    fprintf(stderr, "adding PI child %s to %s\n", target, parent->name);
+#endif
+	    xmlAddChild(parent, ret);
+	} else {
+#ifdef DEBUG_SAX_TREE
+	    fprintf(stderr, "adding PI sibling %s to ", target);
+	    xmlDebugDumpOneNode(stderr, parent, 0);
+#endif
+	    xmlAddSibling(parent, ret);
+	}
+    }
+
 }
 
 /**
@@ -770,7 +995,7 @@ checkNamespace(void *ctx, CHAR *namespace)
 		 "End tags %s holds a prefix %s not used by the open tag\n",
 		                 cur->name, namespace);
 	    ctxt->wellFormed = 0;
-	} else if (strcmp(namespace, cur->ns->prefix)) {
+	} else if (xmlStrcmp(namespace, cur->ns->prefix)) {
 	    if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
 		ctxt->sax->error(ctxt, 
     "Start and End tags for %s don't use the same namespaces: %s and %s\n",
@@ -815,12 +1040,58 @@ comment(void *ctx, const CHAR *value)
 {
     xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) ctx;
     xmlNodePtr ret;
+    xmlNodePtr parent = ctxt->node;
 
 #ifdef DEBUG_SAX
     fprintf(stderr, "SAX.comment(%s)\n", value);
 #endif
     ret = xmlNewDocComment(ctxt->myDoc, value);
+    if (ret == NULL) return;
+
+    if (ctxt->myDoc->root == NULL) {
+#ifdef DEBUG_SAX_TREE
+	    fprintf(stderr, "Setting comment as root\n");
+#endif
+        ctxt->myDoc->root = ret;
+    } else if (parent == NULL) {
+        parent = ctxt->myDoc->root;
+    }
+    if (parent != NULL) {
+        if (parent->type == XML_ELEMENT_NODE) {
+#ifdef DEBUG_SAX_TREE
+	    fprintf(stderr, "adding comment child to %s\n", parent->name);
+#endif
+	    xmlAddChild(parent, ret);
+	} else {
+#ifdef DEBUG_SAX_TREE
+	    fprintf(stderr, "adding comment sibling to ");
+	    xmlDebugDumpOneNode(stderr, parent, 0);
+#endif
+	    xmlAddSibling(parent, ret);
+	}
+    }
+}
+
+/**
+ * cdataBlock:
+ * @ctx: the user data (XML parser context)
+ * @value:  The pcdata content
+ * @len:  the block length
+ *
+ * called when a pcdata block has been parsed
+ */
+void
+cdataBlock(void *ctx, const CHAR *value, int len)
+{
+    xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) ctx;
+    xmlNodePtr ret;
+
+#ifdef DEBUG_SAX
+    fprintf(stderr, "SAX.pcdata(%.10s, %d)\n", value, len);
+#endif
+    ret = xmlNewCDataBlock(ctxt->myDoc, value, len);
     xmlAddChild(ctxt->node, ret);
+    /* !!!!! merges */
 }
 
 /*
@@ -851,6 +1122,8 @@ xmlSAXHandler xmlDefaultSAXHandler = {
     xmlParserWarning,
     xmlParserError,
     xmlParserError,
+    getParameterEntity,
+    cdataBlock,
 };
 
 /**
@@ -867,6 +1140,7 @@ xmlDefaultSAXHandlerInit(void)
     xmlDefaultSAXHandler.hasExternalSubset = hasExternalSubset;
     xmlDefaultSAXHandler.resolveEntity = resolveEntity;
     xmlDefaultSAXHandler.getEntity = getEntity;
+    xmlDefaultSAXHandler.getParameterEntity = getParameterEntity;
     xmlDefaultSAXHandler.entityDecl = entityDecl;
     xmlDefaultSAXHandler.attributeDecl = attributeDecl;
     xmlDefaultSAXHandler.elementDecl = elementDecl;
@@ -879,6 +1153,7 @@ xmlDefaultSAXHandlerInit(void)
     xmlDefaultSAXHandler.endElement = endElement;
     xmlDefaultSAXHandler.reference = reference;
     xmlDefaultSAXHandler.characters = characters;
+    xmlDefaultSAXHandler.cdataBlock = cdataBlock;
     xmlDefaultSAXHandler.ignorableWhitespace = ignorableWhitespace;
     xmlDefaultSAXHandler.processingInstruction = processingInstruction;
     xmlDefaultSAXHandler.comment = comment;
@@ -915,6 +1190,8 @@ xmlSAXHandler htmlDefaultSAXHandler = {
     xmlParserWarning,
     xmlParserError,
     xmlParserError,
+    getParameterEntity,
+    NULL,
 };
 
 /**
@@ -931,6 +1208,7 @@ htmlDefaultSAXHandlerInit(void)
     htmlDefaultSAXHandler.hasExternalSubset = NULL;
     htmlDefaultSAXHandler.resolveEntity = NULL;
     htmlDefaultSAXHandler.getEntity = getEntity;
+    htmlDefaultSAXHandler.getParameterEntity = NULL;
     htmlDefaultSAXHandler.entityDecl = NULL;
     htmlDefaultSAXHandler.attributeDecl = NULL;
     htmlDefaultSAXHandler.elementDecl = NULL;
@@ -943,6 +1221,7 @@ htmlDefaultSAXHandlerInit(void)
     htmlDefaultSAXHandler.endElement = endElement;
     htmlDefaultSAXHandler.reference = NULL;
     htmlDefaultSAXHandler.characters = characters;
+    htmlDefaultSAXHandler.cdataBlock = NULL;
     htmlDefaultSAXHandler.ignorableWhitespace = ignorableWhitespace;
     htmlDefaultSAXHandler.processingInstruction = NULL;
     htmlDefaultSAXHandler.comment = comment;
