@@ -6,9 +6,16 @@
  * Daniel.Veillard@w3.org
  */
 
+#ifndef WIN32
+#include "config.h"
+#endif
+
 #include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+#include "xmlmemory.h"
 #include "entities.h"
 
 /*
@@ -36,27 +43,24 @@ void xmlFreeEntity(xmlEntityPtr entity) {
     if (entity == NULL) return;
 
     if (entity->name != NULL)
-	free((char *) entity->name);
+	xmlFree((char *) entity->name);
     if (entity->ExternalID != NULL)
-        free((char *) entity->ExternalID);
+        xmlFree((char *) entity->ExternalID);
     if (entity->SystemID != NULL)
-        free((char *) entity->SystemID);
+        xmlFree((char *) entity->SystemID);
     if (entity->content != NULL)
-        free((char *) entity->content);
+        xmlFree((char *) entity->content);
     if (entity->orig != NULL)
-        free((char *) entity->orig);
+        xmlFree((char *) entity->orig);
     memset(entity, -1, sizeof(xmlEntity));
 }
 
 /*
  * xmlAddEntity : register a new entity for an entities table.
- *
- * TODO !!! We should check here that the combination of type
- *          ExternalID and SystemID is valid.
  */
 static void
 xmlAddEntity(xmlEntitiesTablePtr table, const CHAR *name, int type,
-              const CHAR *ExternalID, const CHAR *SystemID, CHAR *content) {
+	  const CHAR *ExternalID, const CHAR *SystemID, const CHAR *content) {
     int i;
     xmlEntityPtr cur;
     int len;
@@ -67,8 +71,19 @@ xmlAddEntity(xmlEntitiesTablePtr table, const CHAR *name, int type,
 	    /*
 	     * The entity is already defined in this Dtd, the spec says to NOT
 	     * override it ... Is it worth a Warning ??? !!!
+	     * Not having a cprinting context this seems hard ...
 	     */
-	    return;
+	    if (((type == XML_INTERNAL_PARAMETER_ENTITY) ||
+	         (type == XML_EXTERNAL_PARAMETER_ENTITY)) &&
+	        ((cur->type == XML_INTERNAL_PARAMETER_ENTITY) ||
+	         (cur->type == XML_EXTERNAL_PARAMETER_ENTITY)))
+		return;
+	    else
+	    if (((type != XML_INTERNAL_PARAMETER_ENTITY) &&
+	         (type != XML_EXTERNAL_PARAMETER_ENTITY)) &&
+	        ((cur->type != XML_INTERNAL_PARAMETER_ENTITY) &&
+	         (cur->type != XML_EXTERNAL_PARAMETER_ENTITY)))
+		return;
 	}
     }
     if (table->nb_entities >= table->max_entities) {
@@ -77,10 +92,10 @@ xmlAddEntity(xmlEntitiesTablePtr table, const CHAR *name, int type,
 	 */
 	table->max_entities *= 2;
 	table->table = (xmlEntityPtr) 
-	    realloc(table->table, table->max_entities * sizeof(xmlEntity));
-	if (table->table) {
+	    xmlRealloc(table->table, table->max_entities * sizeof(xmlEntity));
+	if (table->table == NULL) {
 	    perror("realloc failed");
-	    exit(1);
+	    return;
 	}
     }
     cur = &table->table[table->nb_entities];
@@ -168,7 +183,7 @@ xmlGetPredefinedEntity(const CHAR *name) {
  */
 void
 xmlAddDtdEntity(xmlDocPtr doc, const CHAR *name, int type,
-              const CHAR *ExternalID, const CHAR *SystemID, CHAR *content) {
+	  const CHAR *ExternalID, const CHAR *SystemID, const CHAR *content) {
     xmlEntitiesTablePtr table;
 
     if (doc->extSubset == NULL) {
@@ -197,7 +212,7 @@ xmlAddDtdEntity(xmlDocPtr doc, const CHAR *name, int type,
  */
 void
 xmlAddDocEntity(xmlDocPtr doc, const CHAR *name, int type,
-              const CHAR *ExternalID, const CHAR *SystemID, CHAR *content) {
+	  const CHAR *ExternalID, const CHAR *SystemID, const CHAR *content) {
     xmlEntitiesTablePtr table;
 
     if (doc == NULL) {
@@ -218,6 +233,52 @@ xmlAddDocEntity(xmlDocPtr doc, const CHAR *name, int type,
     xmlAddEntity(table, name, type, ExternalID, SystemID, content);
 }
 
+/**
+ * xmlGetParameterEntity:
+ * @doc:  the document referencing the entity
+ * @name:  the entity name
+ *
+ * Do an entity lookup in the internal and external subsets and
+ * returns the corresponding parameter entity, if found.
+ * 
+ * Returns A pointer to the entity structure or NULL if not found.
+ */
+xmlEntityPtr
+xmlGetParameterEntity(xmlDocPtr doc, const CHAR *name) {
+    int i;
+    xmlEntityPtr cur;
+    xmlEntitiesTablePtr table;
+
+    if ((doc->intSubset != NULL) && (doc->intSubset->entities != NULL)) {
+	table = (xmlEntitiesTablePtr) doc->intSubset->entities;
+	for (i = 0;i < table->nb_entities;i++) {
+	    cur = &table->table[i];
+	    if (((cur->type ==  XML_INTERNAL_PARAMETER_ENTITY) ||
+	         (cur->type ==  XML_EXTERNAL_PARAMETER_ENTITY)) &&
+		(!xmlStrcmp(cur->name, name))) return(cur);
+	}
+    }
+    if ((doc->extSubset != NULL) && (doc->extSubset->entities != NULL)) {
+	table = (xmlEntitiesTablePtr) doc->extSubset->entities;
+	for (i = 0;i < table->nb_entities;i++) {
+	    cur = &table->table[i];
+	    if (((cur->type ==  XML_INTERNAL_PARAMETER_ENTITY) ||
+	         (cur->type ==  XML_EXTERNAL_PARAMETER_ENTITY)) &&
+		(!xmlStrcmp(cur->name, name))) return(cur);
+	}
+    }
+    if ((doc->extSubset != NULL) && (doc->extSubset->entities != NULL)) {
+	table = (xmlEntitiesTablePtr) doc->extSubset->entities;
+	for (i = 0;i < table->nb_entities;i++) {
+	    cur = &table->table[i];
+	    if (((cur->type ==  XML_INTERNAL_PARAMETER_ENTITY) ||
+	         (cur->type ==  XML_EXTERNAL_PARAMETER_ENTITY)) &&
+		(!xmlStrcmp(cur->name, name))) return(cur);
+	}
+    }
+    return(NULL);
+}
+
 /**
  * xmlGetDtdEntity:
  * @doc:  the document referencing the entity
@@ -238,7 +299,9 @@ xmlGetDtdEntity(xmlDocPtr doc, const CHAR *name) {
 	table = (xmlEntitiesTablePtr) doc->extSubset->entities;
 	for (i = 0;i < table->nb_entities;i++) {
 	    cur = &table->table[i];
-	    if (!xmlStrcmp(cur->name, name)) return(cur);
+	    if ((cur->type !=  XML_INTERNAL_PARAMETER_ENTITY) &&
+	        (cur->type !=  XML_EXTERNAL_PARAMETER_ENTITY) &&
+	        (!xmlStrcmp(cur->name, name))) return(cur);
 	}
     }
     return(NULL);
@@ -265,7 +328,18 @@ xmlGetDocEntity(xmlDocPtr doc, const CHAR *name) {
 	table = (xmlEntitiesTablePtr) doc->intSubset->entities;
 	for (i = 0;i < table->nb_entities;i++) {
 	    cur = &table->table[i];
-	    if (!xmlStrcmp(cur->name, name)) return(cur);
+	    if ((cur->type !=  XML_INTERNAL_PARAMETER_ENTITY) &&
+	        (cur->type !=  XML_EXTERNAL_PARAMETER_ENTITY) &&
+	        (!xmlStrcmp(cur->name, name))) return(cur);
+	}
+    }
+    if ((doc->extSubset != NULL) && (doc->extSubset->entities != NULL)) {
+	table = (xmlEntitiesTablePtr) doc->extSubset->entities;
+	for (i = 0;i < table->nb_entities;i++) {
+	    cur = &table->table[i];
+	    if ((cur->type !=  XML_INTERNAL_PARAMETER_ENTITY) &&
+	        (cur->type !=  XML_EXTERNAL_PARAMETER_ENTITY) &&
+	        (!xmlStrcmp(cur->name, name))) return(cur);
 	}
     }
     if (xmlPredefinedEntities == NULL)
@@ -273,7 +347,9 @@ xmlGetDocEntity(xmlDocPtr doc, const CHAR *name) {
     table = xmlPredefinedEntities;
     for (i = 0;i < table->nb_entities;i++) {
 	cur = &table->table[i];
-	if (!xmlStrcmp(cur->name, name)) return(cur);
+	if ((cur->type !=  XML_INTERNAL_PARAMETER_ENTITY) &&
+	    (cur->type !=  XML_EXTERNAL_PARAMETER_ENTITY) &&
+	    (!xmlStrcmp(cur->name, name))) return(cur);
     }
 
     return(NULL);
@@ -290,16 +366,13 @@ xmlGetDocEntity(xmlDocPtr doc, const CHAR *name) {
 
 /*
  * A buffer used for converting entities to their equivalent and back.
- *
- * TODO: remove this, this helps performances but forbid reentrancy in a 
- *       stupid way.
  */
 static int buffer_size = 0;
 static CHAR *buffer = NULL;
 
 void growBuffer(void) {
     buffer_size *= 2;
-    buffer = (CHAR *) realloc(buffer, buffer_size * sizeof(CHAR));
+    buffer = (CHAR *) xmlRealloc(buffer, buffer_size * sizeof(CHAR));
     if (buffer == NULL) {
         perror("realloc failed");
         exit(1);
@@ -315,13 +388,11 @@ void growBuffer(void) {
  * Do a global encoding of a string, replacing the predefined entities
  * and non ASCII values with their entities and CharRef counterparts.
  *
- * TODO !!!! Once moved to UTF-8 internal encoding, the encoding of non-ascii
- *           get erroneous.
- *
- * TODO This routine is not reentrant, the interface
- *      should not be modified though.
+ * TODO: remove xmlEncodeEntities, once we are not afraid of breaking binary
+ *       compatibility
  *
  * People must migrate their code to xmlEncodeEntitiesReentrant !
+ * This routine will issue a warning when encountered.
  * 
  * Returns A newly allocated string with the substitution done.
  */
@@ -329,11 +400,18 @@ const CHAR *
 xmlEncodeEntities(xmlDocPtr doc, const CHAR *input) {
     const CHAR *cur = input;
     CHAR *out = buffer;
+    static int warning = 1;
+
+    if (warning) {
+    fprintf(stderr, "Deprecated API xmlEncodeEntities() used\n");
+    fprintf(stderr, "   change code to use xmlEncodeEntitiesReentrant()\n");
+    warning = 0;
+    }
 
     if (input == NULL) return(NULL);
     if (buffer == NULL) {
         buffer_size = 1000;
-        buffer = (CHAR *) malloc(buffer_size * sizeof(CHAR));
+        buffer = (CHAR *) xmlMalloc(buffer_size * sizeof(CHAR));
 	if (buffer == NULL) {
 	    perror("malloc failed");
             exit(1);
@@ -429,7 +507,7 @@ xmlEncodeEntities(xmlDocPtr doc, const CHAR *input) {
  */
 #define growBufferReentrant() {						\
     buffer_size *= 2;							\
-    buffer = (CHAR *) realloc(buffer, buffer_size * sizeof(CHAR));	\
+    buffer = (CHAR *) xmlRealloc(buffer, buffer_size * sizeof(CHAR));	\
     if (buffer == NULL) {						\
 	perror("realloc failed");					\
 	exit(1);							\
@@ -465,7 +543,7 @@ xmlEncodeEntitiesReentrant(xmlDocPtr doc, const CHAR *input) {
      * allocate an translation buffer.
      */
     buffer_size = 1000;
-    buffer = (CHAR *) malloc(buffer_size * sizeof(CHAR));
+    buffer = (CHAR *) xmlMalloc(buffer_size * sizeof(CHAR));
     if (buffer == NULL) {
 	perror("malloc failed");
 	exit(1);
@@ -568,20 +646,20 @@ xmlCreateEntitiesTable(void) {
     xmlEntitiesTablePtr ret;
 
     ret = (xmlEntitiesTablePtr) 
-         malloc(sizeof(xmlEntitiesTable));
+         xmlMalloc(sizeof(xmlEntitiesTable));
     if (ret == NULL) {
-        fprintf(stderr, "xmlCreateEntitiesTable : malloc(%ld) failed\n",
+        fprintf(stderr, "xmlCreateEntitiesTable : xmlMalloc(%ld) failed\n",
 	        (long)sizeof(xmlEntitiesTable));
         return(NULL);
     }
     ret->max_entities = XML_MIN_ENTITIES_TABLE;
     ret->nb_entities = 0;
     ret->table = (xmlEntityPtr ) 
-         malloc(ret->max_entities * sizeof(xmlEntity));
+         xmlMalloc(ret->max_entities * sizeof(xmlEntity));
     if (ret == NULL) {
-        fprintf(stderr, "xmlCreateEntitiesTable : malloc(%ld) failed\n",
+        fprintf(stderr, "xmlCreateEntitiesTable : xmlMalloc(%ld) failed\n",
 	        ret->max_entities * (long)sizeof(xmlEntity));
-	free(ret);
+	xmlFree(ret);
         return(NULL);
     }
     return(ret);
@@ -602,8 +680,8 @@ xmlFreeEntitiesTable(xmlEntitiesTablePtr table) {
     for (i = 0;i < table->nb_entities;i++) {
         xmlFreeEntity(&table->table[i]);
     }
-    free(table->table);
-    free(table);
+    xmlFree(table->table);
+    xmlFree(table);
 }
 
 /**
@@ -620,16 +698,16 @@ xmlCopyEntitiesTable(xmlEntitiesTablePtr table) {
     xmlEntityPtr cur, ent;
     int i;
 
-    ret = (xmlEntitiesTablePtr) malloc(sizeof(xmlEntitiesTable));
+    ret = (xmlEntitiesTablePtr) xmlMalloc(sizeof(xmlEntitiesTable));
     if (ret == NULL) {
         fprintf(stderr, "xmlCopyEntitiesTable: out of memory !\n");
 	return(NULL);
     }
-    ret->table = (xmlEntityPtr) malloc(table->max_entities *
+    ret->table = (xmlEntityPtr) xmlMalloc(table->max_entities *
                                          sizeof(xmlEntity));
     if (ret->table == NULL) {
         fprintf(stderr, "xmlCopyEntitiesTable: out of memory !\n");
-	free(ret);
+	xmlFree(ret);
 	return(NULL);
     }
     ret->max_entities = table->max_entities;
