@@ -10,9 +10,16 @@
 #define HAVE_FCNTL_H
 #include <io.h>
 #else
-#include <config.h>
+#include "config.h"
 #endif
+
+#include <stdio.h>
+#include <string.h>
+#include <stdarg.h>
+
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
@@ -22,10 +29,10 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include <stdio.h>
-#include <string.h>
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
-#include <stdarg.h>
+#endif
+
 
 #include "parser.h"
 #include "parserInternals.h" /* only for xmlNewInputFromFile() */
@@ -61,6 +68,7 @@ xmlSAXHandler emptySAXHandlerStruct = {
     NULL, /* xmlParserWarning */
     NULL, /* xmlParserError */
     NULL, /* xmlParserError */
+    NULL, /* getParameterEntity */
 };
 
 xmlSAXHandlerPtr emptySAXHandler = &emptySAXHandlerStruct;
@@ -104,7 +112,7 @@ static CHAR buffer[] =
  * Returns 1 if true
  */
 int
-isStandaloneDebug(xmlParserCtxtPtr ctxt)
+isStandaloneDebug(void *ctx)
 {
     fprintf(stdout, "SAX.isStandalone()\n");
     return(0);
@@ -119,7 +127,7 @@ isStandaloneDebug(xmlParserCtxtPtr ctxt)
  * Returns 1 if true
  */
 int
-hasInternalSubsetDebug(xmlParserCtxtPtr ctxt)
+hasInternalSubsetDebug(void *ctx)
 {
     fprintf(stdout, "SAX.hasInternalSubset()\n");
     return(0);
@@ -134,7 +142,7 @@ hasInternalSubsetDebug(xmlParserCtxtPtr ctxt)
  * Returns 1 if true
  */
 int
-hasExternalSubsetDebug(xmlParserCtxtPtr ctxt)
+hasExternalSubsetDebug(void *ctx)
 {
     fprintf(stdout, "SAX.hasExternalSubset()\n");
     return(0);
@@ -147,7 +155,7 @@ hasExternalSubsetDebug(xmlParserCtxtPtr ctxt)
  * Does this document has an internal subset
  */
 void
-internalSubsetDebug(xmlParserCtxtPtr ctxt, const CHAR *name,
+internalSubsetDebug(void *ctx, const CHAR *name,
 	       const CHAR *ExternalID, const CHAR *SystemID)
 {
     xmlDtdPtr externalSubset;
@@ -156,7 +164,10 @@ internalSubsetDebug(xmlParserCtxtPtr ctxt, const CHAR *name,
             name, ExternalID, SystemID);
 
     if ((ExternalID != NULL) || (SystemID != NULL)) {
-        externalSubset = xmlSAXParseDTD(debugSAXHandler, ExternalID, SystemID);
+        externalSubset = xmlParseDTD(ExternalID, SystemID);
+	if (externalSubset != NULL) {
+	    xmlFreeDtd(externalSubset);
+	}
     }
 }
 
@@ -175,8 +186,10 @@ internalSubsetDebug(xmlParserCtxtPtr ctxt, const CHAR *name,
  * Returns the xmlParserInputPtr if inlined or NULL for DOM behaviour.
  */
 xmlParserInputPtr
-resolveEntityDebug(xmlParserCtxtPtr ctxt, const CHAR *publicId, const CHAR *systemId)
+resolveEntityDebug(void *ctx, const CHAR *publicId, const CHAR *systemId)
 {
+    xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) ctx;
+
     
     fprintf(stdout, "SAX.resolveEntity(");
     if (publicId != NULL)
@@ -188,7 +201,7 @@ resolveEntityDebug(xmlParserCtxtPtr ctxt, const CHAR *publicId, const CHAR *syst
     else
 	fprintf(stdout, ", )\n");
     if (systemId != NULL) {
-        return(xmlNewInputFromFile(ctxt, systemId));
+        return(xmlNewInputFromFile(ctxt, (char *) systemId));
     }
     return(NULL);
 }
@@ -203,12 +216,28 @@ resolveEntityDebug(xmlParserCtxtPtr ctxt, const CHAR *publicId, const CHAR *syst
  * Returns the xmlParserInputPtr if inlined or NULL for DOM behaviour.
  */
 xmlEntityPtr
-getEntityDebug(xmlParserCtxtPtr ctxt, const CHAR *name)
+getEntityDebug(void *ctx, const CHAR *name)
 {
     fprintf(stdout, "SAX.getEntity(%s)\n", name);
     return(NULL);
 }
 
+/**
+ * getParameterEntityDebug:
+ * @ctxt:  An XML parser context
+ * @name: The entity name
+ *
+ * Get a parameter entity by name
+ *
+ * Returns the xmlParserInputPtr
+ */
+xmlEntityPtr
+getParameterEntityDebug(void *ctx, const CHAR *name)
+{
+    fprintf(stdout, "SAX.getParameterEntity(%s)\n", name);
+    return(NULL);
+}
+
 
 /**
  * entityDeclDebug:
@@ -222,7 +251,7 @@ getEntityDebug(xmlParserCtxtPtr ctxt, const CHAR *name)
  * An entity definition has been parsed
  */
 void
-entityDeclDebug(xmlParserCtxtPtr ctxt, const CHAR *name, int type,
+entityDeclDebug(void *ctx, const CHAR *name, int type,
           const CHAR *publicId, const CHAR *systemId, CHAR *content)
 {
     fprintf(stdout, "SAX.entityDecl(%s, %d, %s, %s, %s)\n",
@@ -238,7 +267,7 @@ entityDeclDebug(xmlParserCtxtPtr ctxt, const CHAR *name, int type,
  * An attribute definition has been parsed
  */
 void
-attributeDeclDebug(xmlParserCtxtPtr ctxt, const CHAR *elem, const CHAR *name,
+attributeDeclDebug(void *ctx, const CHAR *elem, const CHAR *name,
               int type, int def, const CHAR *defaultValue,
 	      xmlEnumerationPtr tree)
 {
@@ -256,7 +285,7 @@ attributeDeclDebug(xmlParserCtxtPtr ctxt, const CHAR *elem, const CHAR *name,
  * An element definition has been parsed
  */
 void
-elementDeclDebug(xmlParserCtxtPtr ctxt, const CHAR *name, int type,
+elementDeclDebug(void *ctx, const CHAR *name, int type,
 	    xmlElementContentPtr content)
 {
     fprintf(stdout, "SAX.elementDecl(%s, %d, ...)\n",
@@ -271,10 +300,9 @@ elementDeclDebug(xmlParserCtxtPtr ctxt, const CHAR *name, int type,
  * @systemId: The system ID of the entity
  *
  * What to do when a notation declaration has been parsed.
- * TODO Not handled currently.
  */
 void
-notationDeclDebug(xmlParserCtxtPtr ctxt, const CHAR *name,
+notationDeclDebug(void *ctx, const CHAR *name,
 	     const CHAR *publicId, const CHAR *systemId)
 {
     fprintf(stdout, "SAX.notationDecl(%s, %s, %s)\n",
@@ -290,10 +318,9 @@ notationDeclDebug(xmlParserCtxtPtr ctxt, const CHAR *name,
  * @notationName: the name of the notation
  *
  * What to do when an unparsed entity declaration is parsed
- * TODO Create an Entity node.
  */
 void
-unparsedEntityDeclDebug(xmlParserCtxtPtr ctxt, const CHAR *name,
+unparsedEntityDeclDebug(void *ctx, const CHAR *name,
 		   const CHAR *publicId, const CHAR *systemId,
 		   const CHAR *notationName)
 {
@@ -311,7 +338,7 @@ unparsedEntityDeclDebug(xmlParserCtxtPtr ctxt, const CHAR *name,
  * Everything is available on the context, so this is useless in our case.
  */
 void
-setDocumentLocatorDebug(xmlParserCtxtPtr ctxt, xmlSAXLocatorPtr loc)
+setDocumentLocatorDebug(void *ctx, xmlSAXLocatorPtr loc)
 {
     fprintf(stdout, "SAX.setDocumentLocator()\n");
 }
@@ -323,7 +350,7 @@ setDocumentLocatorDebug(xmlParserCtxtPtr ctxt, xmlSAXLocatorPtr loc)
  * called when the document start being processed.
  */
 void
-startDocumentDebug(xmlParserCtxtPtr ctxt)
+startDocumentDebug(void *ctx)
 {
     fprintf(stdout, "SAX.startDocument()\n");
 }
@@ -335,7 +362,7 @@ startDocumentDebug(xmlParserCtxtPtr ctxt)
  * called when the document end has been detected.
  */
 void
-endDocumentDebug(xmlParserCtxtPtr ctxt)
+endDocumentDebug(void *ctx)
 {
     fprintf(stdout, "SAX.endDocument()\n");
 }
@@ -346,10 +373,9 @@ endDocumentDebug(xmlParserCtxtPtr ctxt)
  * @name:  The element name
  *
  * called when an opening tag has been processed.
- * TODO We currently have a small pblm with the arguments ...
  */
 void
-startElementDebug(xmlParserCtxtPtr ctxt, const CHAR *name, const CHAR **atts)
+startElementDebug(void *ctx, const CHAR *name, const CHAR **atts)
 {
     int i;
 
@@ -371,7 +397,7 @@ startElementDebug(xmlParserCtxtPtr ctxt, const CHAR *name, const CHAR **atts)
  * called when the end of an element has been detected.
  */
 void
-endElementDebug(xmlParserCtxtPtr ctxt, const CHAR *name)
+endElementDebug(void *ctx, const CHAR *name)
 {
     fprintf(stdout, "SAX.endElement(%s)\n", (char *) name);
 }
@@ -386,9 +412,14 @@ endElementDebug(xmlParserCtxtPtr ctxt, const CHAR *name)
  * Question: how much at a time ???
  */
 void
-charactersDebug(xmlParserCtxtPtr ctxt, const CHAR *ch, int len)
+charactersDebug(void *ctx, const CHAR *ch, int len)
 {
-    fprintf(stdout, "SAX.characters(%.30s, %d)\n", (char *) ch, len);
+    int i;
+
+    fprintf(stdout, "SAX.characters(");
+    for (i = 0;(i < len) && (i < 30);i++)
+	fprintf(stdout, "%c", ch[i]);
+    fprintf(stdout, ", %d)\n", len);
 }
 
 /**
@@ -399,7 +430,7 @@ charactersDebug(xmlParserCtxtPtr ctxt, const CHAR *ch, int len)
  * called when an entity reference is detected. 
  */
 void
-referenceDebug(xmlParserCtxtPtr ctxt, const CHAR *name)
+referenceDebug(void *ctx, const CHAR *name)
 {
     fprintf(stdout, "SAX.reference(%s)\n", name);
 }
@@ -415,7 +446,7 @@ referenceDebug(xmlParserCtxtPtr ctxt, const CHAR *name)
  * Question: how much at a time ???
  */
 void
-ignorableWhitespaceDebug(xmlParserCtxtPtr ctxt, const CHAR *ch, int len)
+ignorableWhitespaceDebug(void *ctx, const CHAR *ch, int len)
 {
     fprintf(stdout, "SAX.ignorableWhitespace(%.30s, %d)\n",
             (char *) ch, len);
@@ -431,7 +462,7 @@ ignorableWhitespaceDebug(xmlParserCtxtPtr ctxt, const CHAR *ch, int len)
  * A processing instruction has been parsed.
  */
 void
-processingInstructionDebug(xmlParserCtxtPtr ctxt, const CHAR *target,
+processingInstructionDebug(void *ctx, const CHAR *target,
                       const CHAR *data)
 {
     fprintf(stdout, "SAX.processingInstruction(%s, %s)\n",
@@ -446,7 +477,7 @@ processingInstructionDebug(xmlParserCtxtPtr ctxt, const CHAR *target,
  * A comment has been parsed.
  */
 void
-commentDebug(xmlParserCtxtPtr ctxt, const CHAR *value)
+commentDebug(void *ctx, const CHAR *value)
 {
     fprintf(stdout, "SAX.comment(%s)\n", value);
 }
@@ -461,7 +492,7 @@ commentDebug(xmlParserCtxtPtr ctxt, const CHAR *value)
  * extra parameters.
  */
 void
-warningDebug(xmlParserCtxtPtr ctxt, const char *msg, ...)
+warningDebug(void *ctx, const char *msg, ...)
 {
     va_list args;
 
@@ -481,7 +512,7 @@ warningDebug(xmlParserCtxtPtr ctxt, const char *msg, ...)
  * extra parameters.
  */
 void
-errorDebug(xmlParserCtxtPtr ctxt, const char *msg, ...)
+errorDebug(void *ctx, const char *msg, ...)
 {
     va_list args;
 
@@ -501,7 +532,7 @@ errorDebug(xmlParserCtxtPtr ctxt, const char *msg, ...)
  * extra parameters.
  */
 void
-fatalErrorDebug(xmlParserCtxtPtr ctxt, const char *msg, ...)
+fatalErrorDebug(void *ctx, const char *msg, ...)
 {
     va_list args;
 
@@ -536,6 +567,7 @@ xmlSAXHandler debugSAXHandlerStruct = {
     warningDebug,
     errorDebug,
     fatalErrorDebug,
+    getParameterEntityDebug,
 };
 
 xmlSAXHandlerPtr debugSAXHandler = &debugSAXHandlerStruct;
