@@ -10,9 +10,15 @@
 #define HAVE_FCNTL_H
 #include <io.h>
 #else
-#include <config.h>
+#include "config.h"
 #endif
+
+#include <stdio.h>
+#include <string.h>
+
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
@@ -22,9 +28,10 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include <stdio.h>
-#include <string.h>
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+
 
 #include "xpath.h"
 #include "tree.h"
@@ -81,7 +88,14 @@ void xmlXPAthDebugDumpNodeSet(FILE *output, xmlNodeSetPtr cur) {
     fprintf(output, "Set contains %d nodes:\n", cur->nodeNr);
     for (i = 0;i < cur->nodeNr;i++) {
         fprintf(output, "%d", i + 1);
-	xmlDebugDumpOneNode(output, cur->nodeTab[i], 2);
+	if (cur->nodeTab[i] == NULL)
+	    fprintf(output, " NULL\n");
+	else if (cur->nodeTab[i]->type == XML_DOCUMENT_NODE)
+	    fprintf(output, " /\n");
+	else if (cur->nodeTab[i]->type == XML_ATTRIBUTE_NODE)
+	    xmlDebugDumpAttr(output, (xmlAttrPtr)cur->nodeTab[i], 2);
+	else
+	    xmlDebugDumpOneNode(output, cur->nodeTab[i], 2);
     }
 }
 
@@ -118,13 +132,14 @@ void testXPath(const char *str) {
     xmlXPathObjectPtr res;
     xmlXPathContextPtr ctxt;
     
-    ctxt = xmlXPathNewContext(document, NULL, NULL, NULL);
+    ctxt = xmlXPathNewContext(document);
     if (expr)
-	res = xmlXPathEvalExpression(str, ctxt);
+	res = xmlXPathEvalExpression(BAD_CAST str, ctxt);
     else
-	res = xmlXPathEval(str, ctxt);
+	res = xmlXPathEval(BAD_CAST str, ctxt);
     xmlXPAthDebugDumpObject(stdout, res);
     xmlXPathFreeObject(res);
+    xmlXPathFreeContext(ctxt);
 }
 
 void testXPathFile(const char *filename) {
