@@ -10,9 +10,15 @@
 #define HAVE_FCNTL_H
 #include <io.h>
 #else
-#include <config.h>
+#include "config.h"
 #endif
+
+#include <stdio.h>
+#include <string.h>
+
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
@@ -22,10 +28,11 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include <stdio.h>
-#include <string.h>
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
 
+#include "xmlmemory.h"
 #include "parser.h"
 #include "tree.h"
 #include "debugXML.h"
@@ -34,6 +41,11 @@ static int debug = 0;
 static int copy = 0;
 static int recovery = 0;
 static int noent = 0;
+static int noout = 0;
+static int valid = 0;
+static int repeat = 0;
+
+extern int xmlDoValidityCheckingDefaultValue;
 
 /*
  * Note: there is a couple of errors introduced on purpose.
@@ -71,18 +83,20 @@ int treeTest(void) {
     /*
      * build a fake XML document
      */
-    doc = xmlNewDoc("1.0");
-    doc->root = xmlNewDocNode(doc, NULL, "EXAMPLE", NULL);
-    xmlSetProp(doc->root, "prop1", "gnome is great");
-    xmlSetProp(doc->root, "prop2", "&linux; too");
-    xmlSetProp(doc->root, "emptyprop", "");
-    tree = xmlNewChild(doc->root, NULL, "head", NULL);
-    subtree = xmlNewChild(tree, NULL, "title", "Welcome to Gnome");
-    tree = xmlNewChild(doc->root, NULL, "chapter", NULL);
-    subtree = xmlNewChild(tree, NULL, "title", "The Linux adventure");
-    subtree = xmlNewChild(tree, NULL, "p", "bla bla bla ...");
-    subtree = xmlNewChild(tree, NULL, "image", NULL);
-    xmlSetProp(subtree, "href", "linus.gif");
+    doc = xmlNewDoc(BAD_CAST "1.0");
+    doc->root = xmlNewDocNode(doc, NULL, BAD_CAST "EXAMPLE", NULL);
+    xmlSetProp(doc->root, BAD_CAST "prop1", BAD_CAST "gnome is great");
+    xmlSetProp(doc->root, BAD_CAST "prop2", BAD_CAST "&linux; too");
+    xmlSetProp(doc->root, BAD_CAST "emptyprop", BAD_CAST "");
+    tree = xmlNewChild(doc->root, NULL, BAD_CAST "head", NULL);
+    subtree = xmlNewChild(tree, NULL, BAD_CAST "title",
+                          BAD_CAST "Welcome to Gnome");
+    tree = xmlNewChild(doc->root, NULL, BAD_CAST "chapter", NULL);
+    subtree = xmlNewChild(tree, NULL, BAD_CAST "title",
+                          BAD_CAST "The Linux adventure");
+    subtree = xmlNewChild(tree, NULL, BAD_CAST "p", BAD_CAST "bla bla bla ...");
+    subtree = xmlNewChild(tree, NULL, BAD_CAST "image", NULL);
+    xmlSetProp(subtree, BAD_CAST "href", BAD_CAST "linus.gif");
 
     /*
      * test intermediate copy if needed.
@@ -96,7 +110,8 @@ int treeTest(void) {
     /*
      * print it.
      */
-    xmlDocDump(stdout, doc);
+    if (noout == 0)
+	xmlDocDump(stdout, doc);
 
     /*
      * free it.
@@ -128,10 +143,12 @@ void parseAndPrintFile(char *filename) {
     /*
      * print it.
      */
-    if (!debug)
-	xmlDocDump(stdout, doc);
-    else
-        xmlDebugDumpDocument(stdout, doc);
+    if (noout == 0) {
+	if (!debug)
+	    xmlDocDump(stdout, doc);
+	else
+	    xmlDebugDumpDocument(stdout, doc);
+    }
 
     /*
      * free it.
@@ -174,7 +191,7 @@ void parseAndPrintBuffer(CHAR *buf) {
 }
 
 int main(int argc, char **argv) {
-    int i;
+    int i, count;
     int files = 0;
 
     for (i = 1; i < argc ; i++) {
@@ -188,23 +205,40 @@ int main(int argc, char **argv) {
 	else if ((!strcmp(argv[i], "-noent")) ||
 	         (!strcmp(argv[i], "--noent")))
 	    noent++;
+	else if ((!strcmp(argv[i], "-noout")) ||
+	         (!strcmp(argv[i], "--noout")))
+	    noout++;
+	else if ((!strcmp(argv[i], "-valid")) ||
+	         (!strcmp(argv[i], "--valid")))
+	    valid++;
+	else if ((!strcmp(argv[i], "-repeat")) ||
+	         (!strcmp(argv[i], "--repeat")))
+	    repeat++;
     }
     if (noent != 0) xmlSubstituteEntitiesDefault(1);
+    if (valid != 0) xmlDoValidityCheckingDefaultValue = 1;
     for (i = 1; i < argc ; i++) {
 	if (argv[i][0] != '-') {
-	    parseAndPrintFile(argv[i]);
+	    if (repeat) {
+		for (count = 0;count < 100 * repeat;count++)
+		    parseAndPrintFile(argv[i]);
+	    } else
+		parseAndPrintFile(argv[i]);
 	    files ++;
 	}
     }
     if (files == 0) {
-	printf("Usage : %s [--debug] [--copy] [--recover] [--noent] XMLfiles ...\n",
+	printf("Usage : %s [--debug] [--copy] [--recover] [--noent] [--noout] XMLfiles ...\n",
 	       argv[0]);
 	printf("\tParse the XML files and output the result of the parsing\n");
 	printf("\t--debug : dump a debug tree of the in-memory document\n");
 	printf("\t--copy : used to test the internal copy implementation\n");
 	printf("\t--recover : output what is parsable on broken XmL documents\n");
 	printf("\t--noent : substitute entity references by their value\n");
+	printf("\t--noout : don't output the result\n");
+	printf("\t--repeat : parse the file 100 times, for timing or profiling\n");
     }
+    xmlMemoryDump();
 
     return(0);
 }
