@@ -3,21 +3,30 @@
  *
  * See Copyright for the status of this software.
  *
- * TODO Cleanup the Dump mechanism.
- *
  * Daniel.Veillard@w3.org
  */
 
+#ifdef WIN32
+#define HAVE_FCNTL_H
+#include <io.h>
+#else
 #include "config.h"
+#endif
+
 #include <stdio.h>
-#include <ctype.h>
-#include <stdlib.h>
 #include <string.h> /* for memset() only ! */
 
+#ifdef HAVE_CTYPE_H
+#include <ctype.h>
+#endif
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
 #ifdef HAVE_ZLIB_H
 #include <zlib.h>
 #endif
 
+#include "xmlmemory.h"
 #include "tree.h"
 #include "entities.h"
 #include "valid.h"
@@ -91,7 +100,7 @@ xmlNewNs(xmlNodePtr node, const CHAR *href, const CHAR *prefix) {
     /*
      * Allocate a new DTD and fill the fields.
      */
-    cur = (xmlNsPtr) malloc(sizeof(xmlNs));
+    cur = (xmlNsPtr) xmlMalloc(sizeof(xmlNs));
     if (cur == NULL) {
         fprintf(stderr, "xmlNewNs : malloc failed\n");
 	return(NULL);
@@ -141,7 +150,7 @@ xmlNewGlobalNs(xmlDocPtr doc, const CHAR *href, const CHAR *prefix) {
     /*
      * Allocate a new DTD and fill the fields.
      */
-    cur = (xmlNsPtr) malloc(sizeof(xmlNs));
+    cur = (xmlNsPtr) xmlMalloc(sizeof(xmlNs));
     if (cur == NULL) {
         fprintf(stderr, "xmlNewGlobalNs : malloc failed\n");
 	return(NULL);
@@ -203,10 +212,10 @@ xmlFreeNs(xmlNsPtr cur) {
         fprintf(stderr, "xmlFreeNs : ns == NULL\n");
 	return;
     }
-    if (cur->href != NULL) free((char *) cur->href);
-    if (cur->prefix != NULL) free((char *) cur->prefix);
+    if (cur->href != NULL) xmlFree((char *) cur->href);
+    if (cur->prefix != NULL) xmlFree((char *) cur->prefix);
     memset(cur, -1, sizeof(xmlNs));
-    free(cur);
+    xmlFree(cur);
 }
 
 /**
@@ -253,7 +262,7 @@ xmlNewDtd(xmlDocPtr doc, const CHAR *name,
     /*
      * Allocate a new DTD and fill the fields.
      */
-    cur = (xmlDtdPtr) malloc(sizeof(xmlDtd));
+    cur = (xmlDtdPtr) xmlMalloc(sizeof(xmlDtd));
     if (cur == NULL) {
         fprintf(stderr, "xmlNewDtd : malloc failed\n");
 	return(NULL);
@@ -306,7 +315,7 @@ xmlCreateIntSubset(xmlDocPtr doc, const CHAR *name,
     /*
      * Allocate a new DTD and fill the fields.
      */
-    cur = (xmlDtdPtr) malloc(sizeof(xmlDtd));
+    cur = (xmlDtdPtr) xmlMalloc(sizeof(xmlDtd));
     if (cur == NULL) {
         fprintf(stderr, "xmlNewDtd : malloc failed\n");
 	return(NULL);
@@ -346,9 +355,9 @@ xmlFreeDtd(xmlDtdPtr cur) {
         fprintf(stderr, "xmlFreeDtd : DTD == NULL\n");
 	return;
     }
-    if (cur->name != NULL) free((char *) cur->name);
-    if (cur->SystemID != NULL) free((char *) cur->SystemID);
-    if (cur->ExternalID != NULL) free((char *) cur->ExternalID);
+    if (cur->name != NULL) xmlFree((char *) cur->name);
+    if (cur->SystemID != NULL) xmlFree((char *) cur->SystemID);
+    if (cur->ExternalID != NULL) xmlFree((char *) cur->ExternalID);
     if (cur->notations != NULL)
         xmlFreeNotationTable((xmlNotationTablePtr) cur->notations);
     if (cur->elements != NULL)
@@ -358,7 +367,7 @@ xmlFreeDtd(xmlDtdPtr cur) {
     if (cur->entities != NULL)
         xmlFreeEntitiesTable((xmlEntitiesTablePtr) cur->entities);
     memset(cur, -1, sizeof(xmlDtd));
-    free(cur);
+    xmlFree(cur);
 }
 
 /**
@@ -379,7 +388,7 @@ xmlNewDoc(const CHAR *version) {
     /*
      * Allocate a new document and fill the fields.
      */
-    cur = (xmlDocPtr) malloc(sizeof(xmlDoc));
+    cur = (xmlDocPtr) xmlMalloc(sizeof(xmlDoc));
     if (cur == NULL) {
         fprintf(stderr, "xmlNewDoc : malloc failed\n");
 	return(NULL);
@@ -395,6 +404,8 @@ xmlNewDoc(const CHAR *version) {
     cur->encoding = NULL;
     cur->standalone = -1;
     cur->compression = xmlCompressMode;
+    cur->ids = NULL;
+    cur->refs = NULL;
 #ifndef XML_WITHOUT_CORBA
     cur->_private = NULL;
     cur->vepv = NULL;
@@ -417,15 +428,17 @@ xmlFreeDoc(xmlDocPtr cur) {
 #endif
 	return;
     }
-    free((char *) cur->version);
-    if (cur->name != NULL) free((char *) cur->name);
-    if (cur->encoding != NULL) free((char *) cur->encoding);
-    if (cur->root != NULL) xmlFreeNode(cur->root);
+    if (cur->version != NULL) xmlFree((char *) cur->version);
+    if (cur->name != NULL) xmlFree((char *) cur->name);
+    if (cur->encoding != NULL) xmlFree((char *) cur->encoding);
+    if (cur->root != NULL) xmlFreeNodeList(cur->root);
     if (cur->intSubset != NULL) xmlFreeDtd(cur->intSubset);
     if (cur->extSubset != NULL) xmlFreeDtd(cur->extSubset);
     if (cur->oldNs != NULL) xmlFreeNsList(cur->oldNs);
+    if (cur->ids != NULL) xmlFreeIDTable((xmlIDTablePtr) cur->ids);
+    if (cur->refs != NULL) xmlFreeRefTable((xmlRefTablePtr) cur->refs);
     memset(cur, -1, sizeof(xmlDoc));
-    free(cur);
+    xmlFree(cur);
 }
 
 /**
@@ -501,7 +514,7 @@ xmlStringLenGetNodeList(xmlDocPtr doc, const CHAR *value, int len) {
 		     */
 		    node = xmlNewReference(doc, val);
 		    if (node == NULL) {
-			if (val != NULL) free(val);
+			if (val != NULL) xmlFree(val);
 		        return(ret);
 		    }
 		    if (last == NULL)
@@ -512,7 +525,7 @@ xmlStringLenGetNodeList(xmlDocPtr doc, const CHAR *value, int len) {
 			last = node;
 		    }
 		}
-		free(val);
+		xmlFree(val);
 	    }
 	    cur++;
 	    q = cur;
@@ -612,7 +625,7 @@ xmlStringGetNodeList(xmlDocPtr doc, const CHAR *value) {
 		     */
 		    node = xmlNewReference(doc, val);
 		    if (node == NULL) {
-			if (val != NULL) free(val);
+			if (val != NULL) xmlFree(val);
 		        return(ret);
 		    }
 		    if (last == NULL)
@@ -623,7 +636,7 @@ xmlStringGetNodeList(xmlDocPtr doc, const CHAR *value) {
 			last = node;
 		    }
 		}
-		free(val);
+		xmlFree(val);
 	    }
 	    cur++;
 	    q = cur;
@@ -679,7 +692,7 @@ xmlNodeListGetString(xmlDocPtr doc, xmlNodePtr list, int inLine) {
 		buffer = xmlEncodeEntitiesReentrant(doc, node->content);
 		if (buffer != NULL) {
 		    ret = xmlStrcat(ret, buffer);
-		    free(buffer);
+		    xmlFree(buffer);
 		}
             }
 	} else if (node->type == XML_ENTITY_REF_NODE) {
@@ -730,7 +743,7 @@ xmlNewProp(xmlNodePtr node, const CHAR *name, const CHAR *value) {
     /*
      * Allocate a new property and fill the fields.
      */
-    cur = (xmlAttrPtr) malloc(sizeof(xmlAttr));
+    cur = (xmlAttrPtr) xmlMalloc(sizeof(xmlAttr));
     if (cur == NULL) {
         fprintf(stderr, "xmlNewProp : malloc failed\n");
 	return(NULL);
@@ -738,6 +751,66 @@ xmlNewProp(xmlNodePtr node, const CHAR *name, const CHAR *value) {
 
     cur->type = XML_ATTRIBUTE_NODE;
     cur->node = node; 
+    cur->ns = NULL;
+    cur->name = xmlStrdup(name);
+    if (value != NULL)
+	cur->val = xmlStringGetNodeList(node->doc, value);
+    else 
+	cur->val = NULL;
+#ifndef XML_WITHOUT_CORBA
+    cur->_private = NULL;
+    cur->vepv = NULL;
+#endif
+
+    /*
+     * Add it at the end to preserve parsing order ...
+     */
+    cur->next = NULL;
+    if (node != NULL) {
+	if (node->properties == NULL) {
+	    node->properties = cur;
+	} else {
+	    xmlAttrPtr prev = node->properties;
+
+	    while (prev->next != NULL) prev = prev->next;
+	    prev->next = cur;
+	}
+    }
+    return(cur);
+}
+
+/**
+ * xmlNewNsProp:
+ * @node:  the holding node
+ * @ns:  the namespace
+ * @name:  the name of the attribute
+ * @value:  the value of the attribute
+ *
+ * Create a new property tagged with a namespace and carried by a node.
+ * Returns a pointer to the attribute
+ */
+xmlAttrPtr
+xmlNewNsProp(xmlNodePtr node, xmlNsPtr ns, const CHAR *name,
+           const CHAR *value) {
+    xmlAttrPtr cur;
+
+    if (name == NULL) {
+        fprintf(stderr, "xmlNewProp : name == NULL\n");
+	return(NULL);
+    }
+
+    /*
+     * Allocate a new property and fill the fields.
+     */
+    cur = (xmlAttrPtr) xmlMalloc(sizeof(xmlAttr));
+    if (cur == NULL) {
+        fprintf(stderr, "xmlNewProp : malloc failed\n");
+	return(NULL);
+    }
+
+    cur->type = XML_ATTRIBUTE_NODE;
+    cur->node = node; 
+    cur->ns = ns;
     cur->name = xmlStrdup(name);
     if (value != NULL)
 	cur->val = xmlStringGetNodeList(node->doc, value);
@@ -786,7 +859,7 @@ xmlNewDocProp(xmlDocPtr doc, const CHAR *name, const CHAR *value) {
     /*
      * Allocate a new property and fill the fields.
      */
-    cur = (xmlAttrPtr) malloc(sizeof(xmlAttr));
+    cur = (xmlAttrPtr) xmlMalloc(sizeof(xmlAttr));
     if (cur == NULL) {
         fprintf(stderr, "xmlNewProp : malloc failed\n");
 	return(NULL);
@@ -840,10 +913,58 @@ xmlFreeProp(xmlAttrPtr cur) {
         fprintf(stderr, "xmlFreeProp : property == NULL\n");
 	return;
     }
-    if (cur->name != NULL) free((char *) cur->name);
+    if (cur->name != NULL) xmlFree((char *) cur->name);
     if (cur->val != NULL) xmlFreeNodeList(cur->val);
     memset(cur, -1, sizeof(xmlAttr));
-    free(cur);
+    xmlFree(cur);
+}
+
+/**
+ * xmlNewPI:
+ * @name:  the processing instruction name
+ * @content:  the PI content
+ *
+ * Creation of a processing instruction element.
+ * Returns a pointer to the new node object.
+ */
+xmlNodePtr
+xmlNewPI(const CHAR *name, const CHAR *content) {
+    xmlNodePtr cur;
+
+    if (name == NULL) {
+        fprintf(stderr, "xmlNewPI : name == NULL\n");
+	return(NULL);
+    }
+
+    /*
+     * Allocate a new node and fill the fields.
+     */
+    cur = (xmlNodePtr) xmlMalloc(sizeof(xmlNode));
+    if (cur == NULL) {
+        fprintf(stderr, "xmlNewPI : malloc failed\n");
+	return(NULL);
+    }
+
+    cur->type = XML_PI_NODE;
+    cur->doc = NULL;
+    cur->parent = NULL; 
+    cur->next = NULL;
+    cur->prev = NULL;
+    cur->childs = NULL;
+    cur->last = NULL;
+    cur->properties = NULL;
+    cur->name = xmlStrdup(name);
+    cur->ns = NULL;
+    cur->nsDef = NULL;
+    if (content != NULL)
+	cur->content = xmlStrdup(content);
+    else
+	cur->content = NULL;
+#ifndef XML_WITHOUT_CORBA
+    cur->_private = NULL;
+    cur->vepv = NULL;
+#endif
+    return(cur);
 }
 
 /**
@@ -868,7 +989,7 @@ xmlNewNode(xmlNsPtr ns, const CHAR *name) {
     /*
      * Allocate a new node and fill the fields.
      */
-    cur = (xmlNodePtr) malloc(sizeof(xmlNode));
+    cur = (xmlNodePtr) xmlMalloc(sizeof(xmlNode));
     if (cur == NULL) {
         fprintf(stderr, "xmlNewNode : malloc failed\n");
 	return(NULL);
@@ -906,7 +1027,7 @@ xmlNewNode(xmlNsPtr ns, const CHAR *name) {
  */
 xmlNodePtr
 xmlNewDocNode(xmlDocPtr doc, xmlNsPtr ns,
-                         const CHAR *name, CHAR *content) {
+                         const CHAR *name, const CHAR *content) {
     xmlNodePtr cur;
 
     cur = xmlNewNode(ns, name);
@@ -935,7 +1056,7 @@ xmlNewText(const CHAR *content) {
     /*
      * Allocate a new node and fill the fields.
      */
-    cur = (xmlNodePtr) malloc(sizeof(xmlNode));
+    cur = (xmlNodePtr) xmlMalloc(sizeof(xmlNode));
     if (cur == NULL) {
         fprintf(stderr, "xmlNewText : malloc failed\n");
 	return(NULL);
@@ -976,7 +1097,7 @@ xmlNewReference(xmlDocPtr doc, const CHAR *name) {
     /*
      * Allocate a new node and fill the fields.
      */
-    cur = (xmlNodePtr) malloc(sizeof(xmlNode));
+    cur = (xmlNodePtr) xmlMalloc(sizeof(xmlNode));
     if (cur == NULL) {
         fprintf(stderr, "xmlNewText : malloc failed\n");
 	return(NULL);
@@ -1043,7 +1164,7 @@ xmlNewTextLen(const CHAR *content, int len) {
     /*
      * Allocate a new node and fill the fields.
      */
-    cur = (xmlNodePtr) malloc(sizeof(xmlNode));
+    cur = (xmlNodePtr) xmlMalloc(sizeof(xmlNode));
     if (cur == NULL) {
         fprintf(stderr, "xmlNewText : malloc failed\n");
 	return(NULL);
@@ -1101,7 +1222,7 @@ xmlNewComment(const CHAR *content) {
     /*
      * Allocate a new node and fill the fields.
      */
-    cur = (xmlNodePtr) malloc(sizeof(xmlNode));
+    cur = (xmlNodePtr) xmlMalloc(sizeof(xmlNode));
     if (cur == NULL) {
         fprintf(stderr, "xmlNewComment : malloc failed\n");
 	return(NULL);
@@ -1126,6 +1247,46 @@ xmlNewComment(const CHAR *content) {
     return(cur);
 }
 
+/**
+ * xmlNewCDataBlock:
+ * @doc:  the document
+ * @content:  the CData block content content
+ * @len:  the length of the block
+ *
+ * Creation of a new node containing a CData block.
+ * Returns a pointer to the new node object.
+ */
+xmlNodePtr
+xmlNewCDataBlock(xmlDocPtr doc, const CHAR *content, int len) {
+    xmlNodePtr cur;
+
+    /*
+     * Allocate a new node and fill the fields.
+     */
+    cur = (xmlNodePtr) xmlMalloc(sizeof(xmlNode));
+    if (cur == NULL) {
+        fprintf(stderr, "xmlNewCDataBlock : malloc failed\n");
+	return(NULL);
+    }
+
+    cur->type = XML_CDATA_SECTION_NODE;
+    cur->doc = NULL; 
+    cur->parent = NULL; 
+    cur->prev = NULL; 
+    cur->next = NULL; 
+    cur->childs = NULL; 
+    cur->last = NULL; 
+    cur->properties = NULL; 
+    cur->name = xmlStrdup(xmlStringText);
+    cur->ns = NULL;
+    cur->nsDef = NULL;
+    if ((content != NULL) && (len > 0)) {
+	cur->content = xmlStrndup(content, len);
+    } else 
+	cur->content = NULL;
+    return(cur);
+}
+
 /**
  * xmlNewDocComment:
  * @doc:  the document
@@ -1158,7 +1319,7 @@ xmlNewDocComment(xmlDocPtr doc, const CHAR *content) {
  */
 xmlNodePtr
 xmlNewChild(xmlNodePtr parent, xmlNsPtr ns,
-                       const CHAR *name, CHAR *content) {
+                       const CHAR *name, const CHAR *content) {
     xmlNodePtr cur, prev;
 
     if (parent == NULL) {
@@ -1199,6 +1360,50 @@ xmlNewChild(xmlNodePtr parent, xmlNsPtr ns,
     return(cur);
 }
 
+/**
+ * xmlAddSibling:
+ * @cur:  the child node
+ * @elem:  the new node
+ *
+ * Add a new element to the list of siblings of @cur
+ * Returns the element or NULL in case of error.
+ */
+xmlNodePtr
+xmlAddSibling(xmlNodePtr cur, xmlNodePtr elem) {
+    xmlNodePtr parent;
+
+    if (cur == NULL) {
+        fprintf(stderr, "xmlAddSibling : cur == NULL\n");
+	return(NULL);
+    }
+
+    if (elem == NULL) {
+        fprintf(stderr, "xmlAddSibling : elem == NULL\n");
+	return(NULL);
+    }
+
+    if ((cur->doc != NULL) && (elem->doc != NULL) &&
+        (cur->doc != elem->doc)) {
+	fprintf(stderr, 
+	        "xmlAddSibling: Elements moved to a different document\n");
+    }
+
+    while (cur->next != NULL) cur = cur->next;
+
+    if (elem->doc == NULL)
+	elem->doc = cur->doc; /* the parent may not be linked to a doc ! */
+
+    parent = cur->parent;
+    elem->prev = cur;
+    elem->next = NULL;
+    elem->parent = parent;
+    cur->next = elem;
+    if (parent != NULL)
+	parent->last = elem;
+
+    return(elem);
+}
+
 /**
  * xmlAddChild:
  * @parent:  the parent node
@@ -1246,7 +1451,7 @@ xmlAddChild(xmlNodePtr parent, xmlNodePtr cur) {
 		text->next->prev = text;
 	    parent->childs = text;
 	    UPDATE_LAST_CHILD(parent)
-	    free(parent->content);
+	    xmlFree(parent->content);
 	    parent->content = NULL;
 	}
     }
@@ -1319,11 +1524,11 @@ xmlFreeNode(xmlNodePtr cur) {
     if (cur->childs != NULL) xmlFreeNodeList(cur->childs);
     if (cur->properties != NULL) xmlFreePropList(cur->properties);
     if (cur->type != XML_ENTITY_REF_NODE)
-	if (cur->content != NULL) free(cur->content);
-    if (cur->name != NULL) free((char *) cur->name);
+	if (cur->content != NULL) xmlFree(cur->content);
+    if (cur->name != NULL) xmlFree((char *) cur->name);
     if (cur->nsDef != NULL) xmlFreeNsList(cur->nsDef);
     memset(cur, -1, sizeof(xmlNode));
-    free(cur);
+    xmlFree(cur);
 }
 
 /**
@@ -1411,6 +1616,7 @@ xmlCopyNamespaceList(xmlNsPtr cur) {
 
 /**
  * xmlCopyProp:
+ * @target:  the element where the attribute will be grafted
  * @cur:  the attribute
  *
  * Do a copy of the attribute.
@@ -1418,7 +1624,7 @@ xmlCopyNamespaceList(xmlNsPtr cur) {
  * Returns: a new xmlAttrPtr, or NULL in case of error.
  */
 xmlAttrPtr
-xmlCopyProp(xmlAttrPtr cur) {
+xmlCopyProp(xmlNodePtr target, xmlAttrPtr cur) {
     xmlAttrPtr ret;
 
     if (cur == NULL) return(NULL);
@@ -1427,6 +1633,15 @@ xmlCopyProp(xmlAttrPtr cur) {
     else
 	ret = xmlNewDocProp(NULL, cur->name, NULL);
     if (ret == NULL) return(NULL);
+    
+    if ((cur->ns != NULL) && (target != NULL)) {
+        xmlNsPtr ns;
+
+	ns = xmlSearchNs(target->doc, target, cur->ns->prefix);
+	ret->ns = ns;
+    } else
+        ret->ns = NULL;
+
     if (cur->val != NULL)
 	ret->val = xmlCopyNodeList(cur->val);
     return(ret);
@@ -1434,6 +1649,7 @@ xmlCopyProp(xmlAttrPtr cur) {
 
 /**
  * xmlCopyPropList:
+ * @target:  the element where the attributes will be grafted
  * @cur:  the first attribute
  *
  * Do a copy of an attribute list.
@@ -1441,12 +1657,12 @@ xmlCopyProp(xmlAttrPtr cur) {
  * Returns: a new xmlAttrPtr, or NULL in case of error.
  */
 xmlAttrPtr
-xmlCopyPropList(xmlAttrPtr cur) {
+xmlCopyPropList(xmlNodePtr target, xmlAttrPtr cur) {
     xmlAttrPtr ret = NULL;
     xmlAttrPtr p = NULL,q;
 
     while (cur != NULL) {
-        q = xmlCopyProp(cur);
+        q = xmlCopyProp(target, cur);
 	if (p == NULL) {
 	    ret = p = q;
 	} else {
@@ -1485,7 +1701,7 @@ xmlStaticCopyNode(xmlNodePtr node, xmlDocPtr doc, xmlNodePtr parent,
     /*
      * Allocate a new node and fill the fields.
      */
-    ret = (xmlNodePtr) malloc(sizeof(xmlNode));
+    ret = (xmlNodePtr) xmlMalloc(sizeof(xmlNode));
     if (ret == NULL) {
         fprintf(stderr, "xmlStaticCopyNode : malloc failed\n");
 	return(NULL);
@@ -1517,8 +1733,6 @@ xmlStaticCopyNode(xmlNodePtr node, xmlDocPtr doc, xmlNodePtr parent,
         xmlAddChild(parent, ret);
     
     if (!recursive) return(ret);
-    if (node->properties != NULL)
-        ret->properties = xmlCopyPropList(node->properties);
     if (node->nsDef != NULL)
         ret->nsDef = xmlCopyNamespaceList(node->nsDef);
 
@@ -1546,6 +1760,8 @@ xmlStaticCopyNode(xmlNodePtr node, xmlDocPtr doc, xmlNodePtr parent,
 	    ret->ns = ns;
 	}
     }
+    if (node->properties != NULL)
+        ret->properties = xmlCopyPropList(ret, node->properties);
     if (node->childs != NULL)
         ret->childs = xmlStaticCopyNodeList(node->childs, doc, ret);
     UPDATE_LAST_CHILD(ret)
@@ -1659,9 +1875,6 @@ xmlCopyDtd(xmlDtdPtr dtd) {
     if (dtd->attributes != NULL)
         ret->attributes = (void *) xmlCopyAttributeTable(
 	                    (xmlAttributeTablePtr) dtd->attributes);
-    /*
-     * TODO: support for Element definitions.
-     */
     return(ret);
 }
 
@@ -1683,7 +1896,7 @@ xmlCopyDoc(xmlDocPtr doc, int recursive) {
     ret = xmlNewDoc(doc->version);
     if (ret == NULL) return(NULL);
     if (doc->name != NULL)
-        ret->name = strdup(doc->name);
+        ret->name = xmlMemStrdup(doc->name);
     if (doc->encoding != NULL)
         ret->encoding = xmlStrdup(doc->encoding);
     ret->compression = doc->compression;
@@ -1705,6 +1918,44 @@ xmlCopyDoc(xmlDocPtr doc, int recursive) {
  *									*
  ************************************************************************/
  
+/**
+ * xmlNodeSetLang:
+ * @cur:  the node being changed
+ * @lang:  the langage description
+ *
+ * Searches the language of a node, i.e. the values of the xml:lang
+ * attribute or the one carried by the nearest ancestor.
+ *
+ * Returns a pointer to the lang value, or NULL if not found
+ */
+void
+xmlNodeSetLang(xmlNodePtr cur, const CHAR *lang) {
+    /* TODO xmlNodeSetLang check against the production [33] LanguageID */
+    xmlSetProp(cur, BAD_CAST "xml:lang", lang);
+}
+ 
+/**
+ * xmlNodeGetLang:
+ * @cur:  the node being checked
+ *
+ * Searches the language of a node, i.e. the values of the xml:lang
+ * attribute or the one carried by the nearest ancestor.
+ *
+ * Returns a pointer to the lang value, or NULL if not found
+ */
+const CHAR *
+xmlNodeGetLang(xmlNodePtr cur) {
+    const CHAR *lang;
+
+    while (cur != NULL) {
+        lang = xmlGetProp(cur, BAD_CAST "xml:lang");
+	if (lang != NULL)
+	    return(lang);
+	cur = cur->parent;
+    }
+    return(NULL);
+}
+ 
 /**
  * xmlNodeGetContent:
  * @cur:  the node being read
@@ -1724,16 +1975,26 @@ xmlNodeGetContent(xmlNodePtr cur) {
         case XML_ELEMENT_NODE:
             return(xmlNodeListGetString(cur->doc, cur->childs, 1));
 	    break;
-        case XML_ATTRIBUTE_NODE:
-        case XML_CDATA_SECTION_NODE:
+        case XML_ATTRIBUTE_NODE: {
+	    xmlAttrPtr attr = (xmlAttrPtr) cur;
+	    if (attr->node != NULL)
+		return(xmlNodeListGetString(attr->node->doc, attr->val, 1));
+	    else
+		return(xmlNodeListGetString(NULL, attr->val, 1));
+	    break;
+	}
+        case XML_PI_NODE:
+	    if (cur->content != NULL)
+	        return(xmlStrdup(cur->content));
+	    return(NULL);
         case XML_ENTITY_REF_NODE:
         case XML_ENTITY_NODE:
-        case XML_PI_NODE:
         case XML_COMMENT_NODE:
         case XML_DOCUMENT_NODE:
         case XML_DOCUMENT_TYPE_NODE:
         case XML_NOTATION_NODE:
 	    return(NULL);
+        case XML_CDATA_SECTION_NODE:
         case XML_TEXT_NODE:
 	    if (cur->content != NULL)
 		return(xmlStrdup(cur->content));
@@ -1759,10 +2020,10 @@ xmlNodeSetContent(xmlNodePtr cur, const CHAR *content) {
         case XML_DOCUMENT_FRAG_NODE:
         case XML_ELEMENT_NODE:
 	    if (cur->content != NULL) {
-	        free(cur->content);
+	        xmlFree(cur->content);
 		cur->content = NULL;
 	    }
-	    if (cur->childs != NULL) xmlFreeNode(cur->childs);
+	    if (cur->childs != NULL) xmlFreeNodeList(cur->childs);
 	    cur->childs = xmlStringGetNodeList(cur->doc, content);
 	    UPDATE_LAST_CHILD(cur)
 	    break;
@@ -1774,13 +2035,14 @@ xmlNodeSetContent(xmlNodePtr cur, const CHAR *content) {
         case XML_ENTITY_NODE:
         case XML_PI_NODE:
         case XML_COMMENT_NODE:
-	    if (cur->content != NULL) free(cur->content);
-	    if (cur->childs != NULL) xmlFreeNode(cur->childs);
+	    if (cur->content != NULL) xmlFree(cur->content);
+	    if (cur->childs != NULL) xmlFreeNodeList(cur->childs);
 	    cur->last = cur->childs = NULL;
 	    if (content != NULL)
 		cur->content = xmlStrdup(content);
 	    else 
 		cur->content = NULL;
+	    break;
         case XML_DOCUMENT_NODE:
         case XML_DOCUMENT_TYPE_NODE:
 	    break;
@@ -1807,10 +2069,10 @@ xmlNodeSetContentLen(xmlNodePtr cur, const CHAR *content, int len) {
         case XML_DOCUMENT_FRAG_NODE:
         case XML_ELEMENT_NODE:
 	    if (cur->content != NULL) {
-	        free(cur->content);
+	        xmlFree(cur->content);
 		cur->content = NULL;
 	    }
-	    if (cur->childs != NULL) xmlFreeNode(cur->childs);
+	    if (cur->childs != NULL) xmlFreeNodeList(cur->childs);
 	    cur->childs = xmlStringLenGetNodeList(cur->doc, content, len);
 	    UPDATE_LAST_CHILD(cur)
 	    break;
@@ -1822,19 +2084,20 @@ xmlNodeSetContentLen(xmlNodePtr cur, const CHAR *content, int len) {
         case XML_ENTITY_NODE:
         case XML_PI_NODE:
         case XML_COMMENT_NODE:
-	    if (cur->content != NULL) free(cur->content);
-	    if (cur->childs != NULL) xmlFreeNode(cur->childs);
+	    if (cur->content != NULL) xmlFree(cur->content);
+	    if (cur->childs != NULL) xmlFreeNodeList(cur->childs);
 	    cur->childs = cur->last = NULL;
 	    if (content != NULL)
 		cur->content = xmlStrndup(content, len);
 	    else 
 		cur->content = NULL;
+	    break;
         case XML_DOCUMENT_NODE:
         case XML_DOCUMENT_TYPE_NODE:
 	    break;
         case XML_NOTATION_NODE:
-	    if (cur->content != NULL) free(cur->content);
-	    if (cur->childs != NULL) xmlFreeNode(cur->childs);
+	    if (cur->content != NULL) xmlFree(cur->content);
+	    if (cur->childs != NULL) xmlFreeNodeList(cur->childs);
 	    cur->childs = cur->last = NULL;
 	    if (content != NULL)
 		cur->content = xmlStrndup(content, len);
@@ -1870,7 +2133,7 @@ xmlNodeAddContentLen(xmlNodePtr cur, const CHAR *content, int len) {
 	        if (cur->content != NULL) {
 		    cur->childs = xmlStringGetNodeList(cur->doc, cur->content);
 		    UPDATE_LAST_CHILD(cur)
-		    free(cur->content);
+		    xmlFree(cur->content);
 		    cur->content = NULL;
 		    last = cur->last;
 		}
@@ -1944,6 +2207,60 @@ xmlTextMerge(xmlNodePtr first, xmlNodePtr second) {
     return(first);
 }
 
+/**
+ * xmlGetNsList:
+ * @doc:  the document
+ * @node:  the current node
+ *
+ * Search all the namespace applying to a given element.
+ * Returns an NULL terminated array of all the xmlNsPtr found
+ *         that need to be freed by the caller or NULL if no
+ *         namespace if defined
+ */
+xmlNsPtr *
+xmlGetNsList(xmlDocPtr doc, xmlNodePtr node) {
+    xmlNsPtr cur;
+    xmlNsPtr *ret = NULL;
+    int nbns = 0;
+    int maxns = 10;
+    int i;
+
+    while (node != NULL) {
+	cur = node->nsDef;
+	while (cur != NULL) {
+	    if (ret == NULL) {
+	        ret = (xmlNsPtr *) xmlMalloc((maxns + 1) * sizeof(xmlNsPtr));
+		if (ret == NULL) {
+		    fprintf(stderr, "xmlGetNsList : out of memory!\n");
+		    return(NULL);
+		}
+		ret[nbns] = NULL;
+	    }
+	    for (i = 0;i < nbns;i++) {
+	        if ((cur->prefix == ret[i]->prefix) ||
+		    (!xmlStrcmp(cur->prefix, ret[i]->prefix))) break;
+	    }
+	    if (i >= nbns) {
+	        if (nbns >= maxns) {
+		    maxns *= 2;
+		    ret = (xmlNsPtr *) xmlRealloc(ret,
+		                         (maxns + 1) * sizeof(xmlNsPtr));
+		    if (ret == NULL) {
+			fprintf(stderr, "xmlGetNsList : realloc failed!\n");
+			return(NULL);
+		    }
+		}
+		ret[nbns++] = cur;
+		ret[nbns] = NULL;
+	    }
+
+	    cur = cur->next;
+	}
+	node = node->parent;
+    }
+    return(ret);
+}
+
 /**
  * xmlSearchNs:
  * @doc:  the document
@@ -2037,7 +2354,7 @@ CHAR *xmlGetProp(xmlNodePtr node, const CHAR *name) {
 	    CHAR *ret;
 
 	    ret = xmlNodeListGetString(node->doc, prop->val, 1);
-	    if (ret == NULL) return(xmlStrdup(""));
+	    if (ret == NULL) return(xmlStrdup((CHAR *)""));
 	    return(ret);
         }
 	prop = prop->next;
@@ -2061,7 +2378,7 @@ xmlSetProp(xmlNodePtr node, const CHAR *name, const CHAR *value) {
     while (prop != NULL) {
         if (!xmlStrcmp(prop->name, name)) {
 	    if (prop->val != NULL) 
-	        xmlFreeNode(prop->val);
+	        xmlFreeNodeList(prop->val);
 	    prop->val = NULL;
 	    if (value != NULL)
 		prop->val = xmlStringGetNodeList(node->doc, value);
@@ -2126,17 +2443,17 @@ xmlBufferPtr
 xmlBufferCreate(void) {
     xmlBufferPtr ret;
 
-    ret = (xmlBufferPtr) malloc(sizeof(xmlBuffer));
+    ret = (xmlBufferPtr) xmlMalloc(sizeof(xmlBuffer));
     if (ret == NULL) {
 	fprintf(stderr, "xmlBufferCreate : out of memory!\n");
         return(NULL);
     }
     ret->use = 0;
     ret->size = BASE_BUFFER_SIZE;
-    ret->content = (CHAR *) malloc(ret->size * sizeof(CHAR));
+    ret->content = (CHAR *) xmlMalloc(ret->size * sizeof(CHAR));
     if (ret->content == NULL) {
 	fprintf(stderr, "xmlBufferCreate : out of memory!\n");
-	free(ret);
+	xmlFree(ret);
         return(NULL);
     }
     ret->content[0] = 0;
@@ -2159,10 +2476,43 @@ xmlBufferFree(xmlBufferPtr buf) {
         fprintf(stderr, "xmlBufferFree: buf->content == NULL\n");
     } else {
         memset(buf->content, -1, BASE_BUFFER_SIZE);
-        free(buf->content);
+        xmlFree(buf->content);
     }
     memset(buf, -1, sizeof(xmlBuffer));
-    free(buf);
+    xmlFree(buf);
+}
+
+/**
+ * xmlBufferEmpty:
+ * @buf:  the buffer
+ *
+ * empty a buffer.
+ */
+void
+xmlBufferEmpty(xmlBufferPtr buf) {
+    buf->use = 0;
+    memset(buf->content, -1, buf->size);/* just for debug */
+}
+
+/**
+ * xmlBufferShrink:
+ * @buf:  the buffer to dump
+ * @len:  the number of CHAR to remove
+ *
+ * Remove the beginning of an XML buffer.
+ *
+ * Returns the number of CHAR removed, or -1 in case of failure.
+ */
+int
+xmlBufferShrink(xmlBufferPtr buf, int len) {
+    if (len == 0) return(0);
+    if (len > buf->use) return(-1);
+
+    buf->use -= len;
+    memmove(buf->content, &buf->content[len], buf->use * sizeof(CHAR));
+
+    buf->content[buf->use] = 0;
+    return(len);
 }
 
 /**
@@ -2200,26 +2550,32 @@ xmlBufferDump(FILE *file, xmlBufferPtr buf) {
  */
 void
 xmlBufferAdd(xmlBufferPtr buf, const CHAR *str, int len) {
-    const CHAR *cur;
+    int l;
 
     if (str == NULL) {
         fprintf(stderr, "xmlBufferAdd: str == NULL\n");
 	return;
     }
-    for (cur = str;(len > 0) && (*cur != 0);cur++, len--) {
-        if (buf->use  + 10 >= buf->size) {
-	    CHAR *rebuf;
+    l = xmlStrlen(str);
+    if (l < len) len = l;
+    if (len <= 0) return;
 
-	    buf->size *= 2;
-	    rebuf = (CHAR *) realloc(buf->content, buf->size * sizeof(CHAR));
-	    if (rebuf == NULL) {
-	        fprintf(stderr, "xmlBufferAdd : out of memory!\n");
-		return;
-	    }
-	    buf->content = rebuf;
+    if (buf->use + len + 10 >= buf->size) {
+	CHAR *rebuf;
+
+        buf->size *= 2;
+	if (buf->use + len + 10 > buf->size)
+	    buf->size = buf->use + len + 10;
+	rebuf = (CHAR *) xmlRealloc(buf->content, buf->size * sizeof(CHAR));
+	if (rebuf == NULL) {
+	    fprintf(stderr, "xmlBufferAdd : out of memory!\n");
+	    return;
 	}
-        buf->content[buf->use++] = *cur;
+	buf->content = rebuf;
     }
+    memmove(&buf->content[buf->use], str, len);
+    buf->use += len;
+    buf->content[buf->use] = 0;
 }
 
 /**
@@ -2242,7 +2598,7 @@ xmlBufferCat(xmlBufferPtr buf, const CHAR *str) {
 	    CHAR *rebuf;
 
 	    buf->size *= 2;
-	    rebuf = (CHAR *) realloc(buf->content, buf->size * sizeof(CHAR));
+	    rebuf = (CHAR *) xmlRealloc(buf->content, buf->size * sizeof(CHAR));
 	    if (rebuf == NULL) {
 	        fprintf(stderr, "xmlBufferAdd : out of memory!\n");
 		return;
@@ -2273,7 +2629,7 @@ xmlBufferCCat(xmlBufferPtr buf, const char *str) {
 	    CHAR *rebuf;
 
 	    buf->size *= 2;
-	    rebuf = (CHAR *) realloc(buf->content, buf->size * sizeof(CHAR));
+	    rebuf = (CHAR *) xmlRealloc(buf->content, buf->size * sizeof(CHAR));
 	    if (rebuf == NULL) {
 	        fprintf(stderr, "xmlBufferAdd : out of memory!\n");
 		return;
@@ -2322,11 +2678,8 @@ xmlBufferWriteChar(xmlBufferPtr buf, const char *string) {
  */
 void
 xmlBufferWriteQuotedString(xmlBufferPtr buf, const CHAR *string) {
-    /*
-     * TODO: fix strchr by xmlStrchr to work coreectly on UTF-8 !!!
-     */
-    if (strchr(string, '"')) {
-        if (strchr(string, '\'')) {
+    if (xmlStrchr(string, '"')) {
+        if (xmlStrchr(string, '\'')) {
 	    fprintf(stderr,
  "xmlBufferWriteQuotedString: string contains quote and double-quotes !\n");
 	}
@@ -2467,7 +2820,6 @@ xmlDtdDump(xmlBufferPtr buf, xmlDocPtr doc) {
 	xmlDumpAttributeTable(buf, (xmlAttributeTablePtr) cur->attributes);
     xmlBufferWriteChar(buf, "]");
 
-    /* TODO !!! a lot more things to dump ... */
     xmlBufferWriteChar(buf, ">\n");
 }
 
@@ -2488,12 +2840,16 @@ xmlAttrDump(xmlBufferPtr buf, xmlDocPtr doc, xmlAttrPtr cur) {
 	return;
     }
     xmlBufferWriteChar(buf, " ");
+    if ((cur->ns != NULL) && (cur->ns->prefix != NULL)) {
+        xmlBufferWriteCHAR(buf, cur->ns->prefix);
+	xmlBufferWriteChar(buf, ":");
+    }
     xmlBufferWriteCHAR(buf, cur->name);
     value = xmlNodeListGetString(doc, cur->val, 0);
     if (value) {
 	xmlBufferWriteChar(buf, "=");
 	xmlBufferWriteQuotedString(buf, value);
-	free(value);
+	xmlFree(value);
     } else  {
 	xmlBufferWriteChar(buf, "=\"\"");
     }
@@ -2579,16 +2935,28 @@ xmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level) {
             buffer = xmlEncodeEntitiesReentrant(doc, cur->content);
 	    if (buffer != NULL) {
 		xmlBufferWriteCHAR(buf, buffer);
-		free(buffer);
+		xmlFree(buffer);
 	    }
 	}
 	return;
     }
+    if (cur->type == XML_PI_NODE) {
+	if (cur->content != NULL) {
+	    xmlBufferWriteChar(buf, "<?");
+	    xmlBufferWriteCHAR(buf, cur->name);
+	    if (cur->content != NULL) {
+		xmlBufferWriteChar(buf, " ");
+		xmlBufferWriteCHAR(buf, cur->content);
+	    }
+	    xmlBufferWriteChar(buf, "?>\n");
+	}
+	return;
+    }
     if (cur->type == XML_COMMENT_NODE) {
 	if (cur->content != NULL) {
 	    xmlBufferWriteChar(buf, "<!--");
 	    xmlBufferWriteCHAR(buf, cur->content);
-	    xmlBufferWriteChar(buf, "-->");
+	    xmlBufferWriteChar(buf, "-->\n");
 	}
 	return;
     }
@@ -2598,6 +2966,13 @@ xmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level) {
         xmlBufferWriteChar(buf, ";");
 	return;
     }
+    if (cur->type == XML_CDATA_SECTION_NODE) {
+        xmlBufferWriteChar(buf, "<![CDATA[");
+	if (cur->content != NULL)
+	    xmlBufferWriteCHAR(buf, cur->content);
+        xmlBufferWriteChar(buf, "]]>");
+	return;
+    }
     if (xmlIndentTreeOutput)
 	for (i = 0;i < level;i++)
 	    xmlBufferWriteChar(buf, "  ");
@@ -2625,7 +3000,7 @@ xmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level) {
 	buffer = xmlEncodeEntitiesReentrant(doc, cur->content);
 	if (buffer != NULL) {
 	    xmlBufferWriteCHAR(buf, buffer);
-	    free(buffer);
+	    xmlFree(buffer);
 	}
     }
     if (cur->childs != NULL) {
@@ -2671,12 +3046,18 @@ xmlDocContentDump(xmlBufferPtr buf, xmlDocPtr cur) {
     if (cur->intSubset != NULL)
         xmlDtdDump(buf, cur);
     if (cur->root != NULL) {
+        xmlNodePtr child = cur->root;
+
 	/* global namespace definitions, the old way */
 	if (oldXMLWDcompatibility)
 	    xmlGlobalNsListDump(buf, cur->oldNs);
 	else 
 	    xmlUpgradeOldNs(cur);
-        xmlNodeDump(buf, cur, cur->root, 0);
+	
+	while (child != NULL) {
+	    xmlNodeDump(buf, cur, child, 0);
+	    child = child->next;
+	}
     }
 }
 
@@ -2708,10 +3089,9 @@ xmlDocDumpMemory(xmlDocPtr cur, CHAR**mem, int *size) {
 	return;
     }
     xmlDocContentDump(buf, cur);
-    *mem = buf->content;
+    *mem = xmlStrndup(buf->content, buf->use);
     *size = buf->use;
-    memset(buf, -1, sizeof(xmlBuffer));
-    free(buf);
+    xmlBufferFree(buf);
 }
 
 /**
